#! /usr/bin/env python

import re
import seqdict
import sys
import getopt
import os

version="0.1"
date="Jul 29 2007"

def usage():
    print "Stimulus version", version, date
    print sys.argv[0], "-f <function-file> -t <type-file> -l language "
    print ' ' * len(sys.argv[0]), "-i <input-file> -o <output-file>"
    print ' ' * len(sys.argv[0]), "-h --help -v"

################################################################################
class StimulusError(Exception):
    def __init__(self, message):
        self.msg = message
    def __str__(self):
        return str(self.msg)

################################################################################
class PLexer:
    def __init__(self, stream):
        self.stream=stream
        self.ws_stack=[0]
        self.tokens=[]
        self.lineno=0

    def lineno(self):
        return self.lineno

    def token(self):
        keys=[]

        if (len(self.tokens)>0):
            return self.tokens.pop(0)

        # Read a line, skip empty lines and comments
        while True:
            line=self.stream.readline(); self.lineno = self.lineno+1
            if line=="":
                for k in keys:
                    self.tokens.append( ("key", k) )
                    keys=[]
                while len(self.ws_stack)>0:
                    self.tokens.append( ("dedent", "") )
                    self.ws_stack.pop()
                self.tokens.append( ("eof", "") )
                return self.tokens.pop(0)
            if re.match("^[ \t]*$", line): continue
            if re.match("^[ \t]*#", line): continue
            break

        if line[-1]=="\n": line=line[:(len(line)-1)]
        ws=re.match(r"^[ \t]*", line).span()[1]
        line=line.strip()
        if ws > self.ws_stack[-1]:
            self.tokens.append( ("indent", "") )
            self.ws_stack.append(ws)
        else:
            for k in keys:
                self.tokens.append( ("key", k) )
                keys=[]
            while ws < self.ws_stack[-1]:
                self.ws_stack.pop()
                self.tokens.append( ("dedent", "") )
            if ws != self.ws_stack[-1]:
                print "Bad indentation in line", self.lineno
                exit

        # Ok, we're done with the white space, now let's see
        # whether this line is continued
        while line[-1]=="\\":
            line=line[:(len(line)-1)]
            line=line+"\n  " + self.stream.readline().strip() ; self.lineno=self.lineno+1

        # We have the line now, check whether there is a ':' in it
        line=line.split(":", 1)
        if len(line)>1:
            line[0]=line[0].strip()
            line[1]=line[1].strip()
            if line[0]=="":
                print "Missing keyword in line", self.lineno
                exit
            keys=line[0].split(",")
            keys=[ k.strip() for k in keys ]
            if line[1] == "":
                self.tokens.append( ("key", keys.pop(0)) )
            else:
                for k in keys:
                    self.tokens.append( ("key", k))
                    self.tokens.append( ("indent", "") )
                    self.tokens.append( ("text", line[1]) )
                    self.tokens.append( ("dedent", "") )
        else:
            self.tokens.append( ("text", line[0].strip()) )
            for k in keys:
                self.tokens.append( ("dedent", "") )
                self.tokens.append( ("key", k) )
                self.tokens.append( ("indent", "") )
            keys=[]
                
        if self.tokens:
            return self.tokens.pop(0)

################################################################################
class PParser:
    def parse(self, stream):
        lex=PLexer(stream)
        val=seqdict.seqdict()
        val_stack=[val, None]
        nam_stack=[None, None]
        
        tok=lex.token()
        while not tok[0]=="eof":
            if tok[0]=="indent":
                val_stack.append(None)
                nam_stack.append(None)
            elif tok[0]=="dedent":
                v=val_stack.pop()
                n=nam_stack.pop()
                if n is None:                    
                    val_stack[-1]=v
                else:
                    val_stack[-1][n]=v
            elif tok[0]=="key":
                if not nam_stack[-1] is None:
                    val_stack[-2][nam_stack[-1]]=val_stack[-1]
                if tok[1][-5:]=="-list":
                    val_stack[-1]=seqdict.seqdict()
                    nam_stack[-1]=tok[1][:-5]
                else:
                    val_stack[-1]={}
                    nam_stack[-1]=tok[1]
            elif tok[0]=="text":
                val_stack[-1]=tok[1]
            tok=lex.token()

        return val

################################################################################

def main():
    # Command line arguments
    try:
        optlist, args = getopt.getopt(sys.argv[1:], 't:f:l:i:o:hv', ['help'])
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    types=[]; functions=[]; inputs=[]; languages=[]; outputs=[]; verbose=False

    for o,a in optlist:
        if o in ("-h", "--help"):
            usage()
            sys.exit()
        elif o == "-o":
            outputs.append(a)
        elif o == "-t":
            types.append(a)
        elif o == "-f":
            functions.append(a)        
        elif o == "-l":
            languages.append(a)
        elif o == "-i":
            inputs.append(a)
        elif o =="-v":
            verbose=True

    # Parameter checks
    # Note: the lists might be empty, but languages and outputs must
    # have the same length.
    if len(languages) != len(outputs):
        print "Error: number of languages and output files must match"
        sys.exit(4)
    for l in languages:
        if not l+"CodeGenerator" in globals():
            print "Error: unknown language:", l
            sys.exit(6)
    for f in types:
        if not os.access(f, os.R_OK):
            print "Error: cannot open type file:", f
            sys.exit(5)
    for f in functions:
        if not os.access(f, os.R_OK):
            print "Error: cannot open function file:", f
            sys.exit(5)
    for f in inputs:
        if not os.access(f, os.R_OK):
            print "Error: cannot open input file:", f
            sys.exit(5)
    # TODO: output files are not checked now

    # OK, do the trick:
    for l in range(len(languages)):
        cl=globals()[languages[l]+"CodeGenerator"]
        cg=cl(functions, types)
        cg.generate(inputs, outputs[l])

################################################################################
class CodeGenerator:
    def __init__(self, func, types):
        # Set name
        self.name=str(self.__class__).split(".")[-1]
        self.name=self.name[0:len(self.name)-len("CodeGenerator")]

        # Parse function and type files
        parser=PParser()
        self.func=seqdict.seqdict()
        for f in func:
            ff=open(f)
            newfunc=parser.parse(ff)
            self.func.extend(newfunc)
            ff.close()

        self.types=seqdict.seqdict()
        for t in types:
            ff=open(t)
            newtypes=parser.parse(ff)
            self.types.extend(newtypes)
            ff.close()

        # The default return type is 'ERROR'
        for f in self.func.keys():
            if 'RETURN' not in self.func[f]:
                self.func[f]['RETURN']='ERROR'

    def generate(self, inputs, output):
        out=open(output, "w")
        self.append_inputs(inputs, out)
        for f in self.func.keys():
            if 'FLAGS' in self.func[f]:
                flags=self.func[f]['FLAGS']
                flags=flags.split(",")
                flags=[ flag.strip() for flag in flags ]
            else:
                self.func[f]['FLAGS']=[]
            self.generate_function(f, out)
        out.close()

    def generate_function(self, f, out):
        print "Error: invalid code generator, this method should be overridden"
        sys.exit(1)

    def parse_params(self, function):
        if "PARAMS" not in self.func[function]:
            return seqdict.seqdict()
        
        params=self.func[function]["PARAMS"]
        params=params.split(",")
        params=[ p.strip() for p in params ]
        params=[ p.split(" ", 1) for p in params ]
        for p in range(len(params)):
            if params[p][0] in ['OUT', 'IN', 'INOUT']:
                params[p]=[params[p][0]] + params[p][1].split(" ", 1)
            else:
                params[p]=['IN', params[p][0]]+ params[p][1].split(" ", 1)
            if '=' in params[p][2]:
                params[p]=params[p][:2] + params[p][2].split("=", 1)
        params=[ [ p.strip() for p in pp ] for pp in params ]
        res=seqdict.seqdict()
        for p in params:
            if len(p)==3:
                res[ p[2] ] = { 'mode': p[0], 'type': p[1] }
            else:
                res[ p[2] ] = { 'mode': p[0], 'type': p[1], 'default': p[3] }
        return res

    def parse_deps(self, function):
        if 'DEPS' not in self.func[function]:
            return seqdict.seqdict()

        deps=self.func[function]["DEPS"]
        deps=deps.split(",")
        deps=[ d.strip() for d in deps ]
        deps=[ d.split("ON", 1) for d in deps ]
        deps=[ [ dd.strip() for dd in d ] for d in deps ]
        deps=[ [d[0]] + d[1].split(" ",1) for d in deps ]
        deps=[ [ dd.strip() for dd in d ] for d in deps ]
        res=seqdict.seqdict()
        for d in deps:
            res[ d[0] ] = d[1:]
        return res

    def append_inputs(self, inputs, output):
        for i in inputs:
            ii=open(i)
            str=ii.read()
            while str != "":
                output.write(str)
                str=ii.read()
            ii.close()
        pass    

    def ignore(self, function):
        if 'IGNORE' in self.func[function]:
            ign=self.func[function]['IGNORE']
            ign=ign.split(",")
            ign=[i.strip() for i in ign]
            if self.name in ign: return True
        return False

################################################################################
# GNU R, see http://www.r-project.org
# TODO: free memory when CTRL+C pressed, even on windows
################################################################################

class RNamespaceCodeGenerator(CodeGenerator):
    def __init__(self, func, types):
        CodeGenerator.__init__(self, func, types)

    def generate(self, inputs, output):
        """This is very simple, we include an 'export' line for every
        function which it not to be ignored by the RNamespace language.
        Function names are taken from NAME-R if present, otherwise
        underscores are converted to dots and the leading 'i' (from
        'igraph') is stripped to create the function name,
        ie. igraph_clusters is mapped to graph.clusters."""
        out=open(output, "w")
        self.append_inputs(inputs, out)
        for f in self.func.keys():
            if (self.ignore(f)):
                continue
            name=self.func[f].get("NAME-R", f[1:].replace("_", "."))
            out.write("export(" + name + ")\n")
        out.close()
        

class RRCodeGenerator(CodeGenerator):
    def __init__(self, func, types):
        CodeGenerator.__init__(self, func, types)

    def generate_function(self, function, out):
        
        # Ignore?
        if self.ignore(function):
            return

        name=self.func[function].get("NAME-R", function[1:].replace("_", "."))
        params=self.parse_params(function)
        self.deps=self.parse_deps(function)

        # Check types
        for p in params.keys():
            tname=params[p]['type']
            if not tname in self.types.keys():
                print "Error: Unknown type encountered:", tname
                sys.exit(7)
            params[p].setdefault('mode', 'IN')

        ## Roxygen to export the function
        internal = self.func[function].get("INTERNAL")
        print internal
        if internal is None or internal == 'False':
            out.write("#' @export\n")

        ## Header
        ## do_par handles the translation of a single argument in the
        ## header. Pretty simple, the only difficulty is that we
        ## might need to add default values. Default values are taken
        ## from a language specific dictionary, this is compiled from
        ## the type file(s).
        
        ## So we take all arguments with mode 'IN' or 'INOUT' and
        ## check whether they have a default value. If yes then we 
        ## check if the default value is given in the type file. If
        ## yes then we use the value given there, otherwise the
        ## default value is ignored silently. (Not very nice.)
        
        out.write(name)
        out.write(" <- function(")
        def do_par(pname):
            tname=params[pname]['type']
            t=self.types[tname]
            default=""
            header=pname.replace("_", ".")
            if 'HEADER' in t:
                header=t['HEADER']
            if header:
                header=header.replace("%I%", pname.replace("_", "."))
            else:
                header=""
            if 'default' in params[pname]:
                if 'DEFAULT' in t and params[pname]['default'] in t['DEFAULT']:
                    default="=" + t['DEFAULT'][ params[pname]['default'] ]
                else:
                    default="=" + params[pname]['default']
            header = header + default

            if pname in self.deps.keys():
                deps = self.deps[pname]
                for i in range(len(deps)):
                    header=header.replace("%I"+str(i+1)+"%", deps[i])
            
            return header
            
        head=[ do_par(n) for n,p in params.items()
               if p['mode'] in ['IN','INOUT'] ]
        head=[ h for h in head if h != "" ]
        out.write(", ".join(head))
        out.write(") {\n")

        ## Argument checks, INCONV
        ## We take 'IN' and 'INOUT' mode arguments and if they have an
        ## INCONV field then we use that. This is typically for
        ## argument checks, like we check here that the argument
        ## supplied for a graph is indeed an igraph graph object. We
        ## also covert numeric vectors to 'double' here.

        ## The INCONV fields are simply concatenated by newline
        ## characters.
        out.write("  # Argument checks\n")
        def do_par(pname):
            t=self.types[params[pname]['type']]
            m=params[pname]['mode']
            if m in ['IN', 'INOUT'] and 'INCONV' in t:
                if m in t['INCONV']:
                    res= "  " + t['INCONV'][m]
                else:
                    res="  " + t['INCONV']
            else:
                res=""
            res=res.replace("%I%", pname.replace("_", "."))

            if pname in self.deps.keys():
                deps = self.deps[pname]
                for i in range(len(deps)):
                    res=res.replace("%I"+str(i+1)+"%", deps[i])
            return res

        inconv=[ do_par(n) for n in params.keys() ]
        inconv=[ i for i in inconv if i != "" ]
        out.write("\n".join(inconv)+"\n\n")

        ## Function call
        ## This is a bit more difficult than INCONV. Here we supply
        ## each argument to the .Call function, if the argument has a
        ## 'CALL' field then it is used, otherwise we simply use its
        ## name. 
        ## argument. Note that arguments with empty CALL fields are
        ## completely ignored, so giving an empty CALL field is
        ## different than not giving it at all.

        ## Function call
        def do_par(pname):
            t=self.types[params[pname]['type']]
            call=pname.replace("_", ".")
            if 'CALL' in t:
                call=t['CALL']
                if call:
                    call=call.replace('%I%', pname.replace("_", "."))
                else:
                    call=""
            return call
              
        out.write("  on.exit( .Call(C_R_igraph_finalizer) )\n")
        out.write("  # Function call\n")
        out.write("  res <- .Call(C_R_" + function + ", ")
        call=[ do_par(n) for n,p in params.items() if p['mode'] in ['IN', 'INOUT'] ]
        call=[ c for c in call if c != "" ]
        out.write(", ".join(call))
        out.write(")\n")

        ## Output conversions
        def do_opar(pname, realname=None, iprefix=""):
            if realname is None: realname=pname
            t=self.types[params[pname]['type']]
            mode=params[pname]['mode']
            if 'OUTCONV' in t and mode in t['OUTCONV']:
                outconv="  " + t['OUTCONV'][mode]
            else:
                outconv=""
            outconv=outconv.replace("%I%", iprefix+realname)

            if pname in self.deps.keys():
                deps = self.deps[pname]
                for i in range(len(deps)):
                    outconv=outconv.replace("%I"+str(i+1)+"%", deps[i])
            return re.sub("%I[0-9]+%", "", outconv)

        retpars=[ n for n,p in params.items() if p['mode'] in 
                  ['OUT', 'INOUT'] ]

        if len(retpars) <= 1:
            outconv=[ do_opar(n, "res") for n in params.keys() ]
        else:
            outconv=[ do_opar(n, iprefix="res$") for n in params.keys() ]

        outconv=[ o for o in outconv if o != "" ]

        if len(retpars)==0:
            # returning the return value of the function
            rt=self.types[self.func[function]['RETURN']]
            if 'OUTCONV' in rt:
                retconv="  " + rt['OUTCONV']['OUT']
            else:
                retconv=""
            retconv=retconv.replace("%I%", "res")
            # TODO: %I1% etc, is not handled here!
            ret="\n".join(outconv) + "\n" + retconv + "\n"
        elif len(retpars)==1:
            # returning a single output value
            ret="\n".join(outconv) + "\n"
        else:
            # returning a list of output values
            None
            ret="\n".join(outconv) + "\n"
        out.write(ret)

        ## Some graph attributes to add
        if 'GATTR-R' in self.func[function].keys():
            gattrs=self.func[function]['GATTR-R'].split(',')
            gattrs=[ ga.split(' IS ', 1) for ga in gattrs ]
            sstr="  res <- set.graph.attribute(res, '%s', '%s')\n"
            for ga in gattrs:
                aname=ga[0].strip()
                aval=ga[1].strip().replace("'", "\\'")
                out.write(sstr % (aname, aval))

        ## Add some parameters as graph attributes
        if 'GATTR-PARAM-R' in self.func[function].keys():
            pars=self.func[function]['GATTR-PARAM-R'].split(',')
            pars=[ p.strip().replace("_", ".") for p in pars ]
            sstr="  res <- set.graph.attribute(res, '%s', %s)\n"
            for p in pars:
                out.write(sstr % (p, p))

        ## Set the class if requested
        if 'CLASS-R' in self.func[function].keys():
            myclass=self.func[function]['CLASS-R']
            out.write("  class(res) <- \"" + myclass + "\"\n")

        ## See if there is a postprocessor
        if 'PP-R' in self.func[function].keys():
            pp=self.func[function]['PP-R']
            out.write("  res <- " + pp + "(res)\n")

        out.write("  res\n}\n\n")

class RCCodeGenerator(CodeGenerator):
    def __init__(self, func, types):
        CodeGenerator.__init__(self, func, types)

    def generate_function(self, function, out):

        # Ignore?
        if self.ignore(function):
            return

        params=self.parse_params(function)
        self.deps = self.parse_deps(function)

        # Check types
        for p in params.keys():
            tname=params[p]['type']
            if not tname in self.types.keys():
                print "Error: Unknown type encountered:", tname
                sys.exit(7)
            params[p].setdefault('mode', 'IN')

        ## Compile the output
        ## This code generator is quite difficult, so we use different
        ## functions to generate the approprite chunks and then
        ## compile them together using a simple template.
        ## See the documentation of each chunk below.
        res={}
        res['func']=function
        res['header']=self.chunk_header(function, params)
        res['decl']=self.chunk_declaration(function, params)
        res['inconv']=self.chunk_inconv(function, params)
        res['call']=self.chunk_call(function, params)
        res['outconv']=self.chunk_outconv(function, params)

        # Replace into the template
        text="""
/*-------------------------------------------/
/ %(func)-42s /
/-------------------------------------------*/
%(header)s {
                                        /* Declarations */
%(decl)s
                                        /* Convert input */
%(inconv)s
                                        /* Call igraph */
%(call)s
                                        /* Convert output */
%(outconv)s

  UNPROTECT(1);
  return(result);
}\n""" % res

        out.write(text)
        
    def chunk_header(self, function, params):
        """The header. All functions return with a 'SEXP', so this is
        easy. We just take the 'IN' and 'INOUT' arguments, all will
        have type SEXP, and concatenate them by commas. The function name
        is created by prefixing the original name with 'R_'."""
        def do_par(pname):
            t=self.types[params[pname]['type']]
            if 'HEADER' in t:
                if t['HEADER']:
                    return t['HEADER'].replace("%I%", pname)
                else:
                    return ""
            else:
                return pname
            
        inout=[ do_par(n) for n,p in params.items()
                if p['mode'] in ['IN','INOUT'] ]
        inout=[ "SEXP " + n for n in inout if n != "" ]
        return "SEXP R_" + function + "(" + ", ".join(inout) + ")"
        
    def chunk_declaration(self, function, params):
        """There are a couple of things to declare. First a C type is
        needed for every argument, these will be supplied in the C
        igraph call. Then, all 'OUT' arguments need a SEXP variable as
        well, the result will be stored here. The return type
        of the C function also needs to be declared, that comes
        next. The result and names SEXP variables will contain the
        final result, these are last. ('names' is not always used, but
        it is easier to always declare it.)
        """
        def do_par(pname):
            cname="c_"+pname
            t=self.types[params[pname]['type']]
            if 'DECL' in t:
                decl="  " + t['DECL']
            elif 'CTYPE' in t:
                ctype = t['CTYPE']
                if type(ctype)==dict:
                    mode=params[pname]['mode']
                    decl="  " + ctype[mode] + " " + cname + ";"
                else:
                    decl="  " + ctype + " " + cname + ";"
            else:
                decl=""
            return decl.replace("%C%", cname).replace("%I%", pname)

        inout=[ do_par(n) for n in params.keys() ]
        out=[ "  SEXP "+n+";" for n,p in params.items()
              if p['mode']=='OUT' ]

        retpars=[ n for n,p in params.items() if p['mode'] in 
                  ['OUT', 'INOUT'] ]

        rt=self.types[self.func[function]['RETURN']]    
        if 'DECL' in rt:
            retdecl="  " + rt['DECL']
        elif 'CTYPE' in rt and len(retpars)==0:
            ctype=rt['CTYPE']
            if type(ctype)==dict:
                mode=params[pname]['mode']
                retdecl="  " + ctype[mode] + " " + "c_result;"
            else:
                retdecl="  " + rt['CTYPE'] + " c_result;"
        else:
            retdecl=""

        if len(retpars)<=1:
            res = "\n".join(inout + out + [retdecl] + ["  SEXP result;"])
        else:
            res = "\n".join(inout + out + [retdecl] + 
                            ["  SEXP result, names;"])
        return res

    def chunk_inconv(self, function, params):
        """Input conversions. Not only for types with mode 'IN' and
        'INOUT', eg. for 'OUT' vector types we need to allocate the
        required memory here, do all the initializations, etc. Types
        without INCONV fields are ignored. The usual %C%, %I% is
        performed at the end.
        """
        def do_par(pname):
            cname="c_"+pname
            t=self.types[params[pname]['type']]
            mode=params[pname]['mode']
            if 'INCONV' in t and mode in t['INCONV']:
                inconv="  " + t['INCONV'][mode]
            else:
                inconv=""

            if pname in self.deps.keys():
                deps = self.deps[pname]
                for i in range(len(deps)):
                    inconv=inconv.replace("%C"+str(i+1)+"%", "c_"+deps[i])
                
            return inconv.replace("%C%", cname).replace("%I%", pname)

        inconv=[ do_par(n) for n in params.keys() ]
        inconv=[ i for i in inconv if i != "" ]

        return "\n".join(inconv)

    def chunk_call(self, function, params):
        """Every single argument is included, independently of their
        mode. If a type has a 'CALL' field then that is used after the
        usual %C% and %I% substitutions, otherwise the standard 'c_'
        prefixed C argument name is used.
        """
        def docall(t, n):
            if type(t)==dict:
                mode=params[n]['mode']
                if mode in t:
                    return t[mode]
                else:
                    return ""
            else:
                return t
        
        types=[ self.types[params[n]['type']] for n in params.keys() ]
        call=map( lambda t, n: docall(t.get('CALL', "c_"+n), n), types, 
                  params.keys() )
        call=map( lambda c, n: c.replace("%C%", "c_"+n).replace("%I%", n),
                  call, params.keys() )
        retpars=[ n for n,p in params.items() if p['mode'] in 
                  ['OUT', 'INOUT'] ]
        call=[ c for c in call if c != "" ]
        res="  " + function + "(" + ", ".join(call) + ");\n"
        if len(retpars)==0:
            res="  c_result=" + res
        return res

    def chunk_outconv(self, function, params):
        """The output conversions, this is quite difficult. A function
        may report its results in two ways: by returning it directly
        or by setting a variable to which a pointer was passed. igraph
        usually uses the latter and returns error codes, except for
        some simple functions like 'igraph_vcount()' which cannot
        fail.

        First we add the output conversion for all types. This is
        easy. Note that even 'IN' arguments may have output
        conversion, eg. this is the place to free memory allocated to
        them in the 'INCONV' part.

        Then we check how many 'OUT' or 'INOUT' arguments we
        have. There are three cases. If there is a single such
        argument then that is already converted and we need to return
        that. If there is no such argument then the output of the
        function was returned, so we perform the output conversion for
        the returned type and this will be the result. If there are
        more than one 'OUT' and 'INOUT' arguments then they are
        collected in a named list. The names come from the argument
        names.
        """
        def do_par(pname):
            cname="c_"+pname
            t=self.types[params[pname]['type']]
            mode=params[pname]['mode']
            if 'OUTCONV' in t and mode in t['OUTCONV']:
                outconv="  " + t['OUTCONV'][mode]
            else:
                outconv=""

            if pname in self.deps.keys():
                deps = self.deps[pname]
                for i in range(len(deps)):
                    outconv=outconv.replace("%C"+str(i+1)+"%", "c_"+deps[i])
            return outconv.replace("%C%", cname).replace("%I%", pname)

        outconv=[ do_par(n) for n in params.keys() ]
        outconv=[ o for o in outconv if o != "" ]

        retpars=[ n for n,p in params.items() if p['mode'] in ['OUT', 'INOUT'] ]
        if len(retpars)==0:
            # return the return value of the function
            rt=self.types[self.func[function]['RETURN']]
            if 'OUTCONV' in rt:
                retconv="  " + rt['OUTCONV']['OUT']
            else:
                retconv=""
            retconv=retconv.replace("%C%", "c_result").replace("%I%", "result")
            ret="\n".join(outconv) + "\n" + retconv
        elif len(retpars)==1:
            # return the single output value
            retconv="  result=" + retpars[0] + ";"
            ret="\n".join(outconv) + "\n" + retconv
        else:
            # create a list of output values
            sets=map ( lambda c, n: "  SET_VECTOR_ELT(result, "+str(c)+", "+n+");",
                       range(len(retpars)), retpars )
            names=map ( lambda c, n: "  SET_STRING_ELT(names, "+str(c)+
                        ", CREATE_STRING_VECTOR(\""+n+"\"));",
                        range(len(retpars)), retpars )
            ret="\n".join(["  PROTECT(result=NEW_LIST(" + str(len(retpars)) + "));",
                           "  PROTECT(names=NEW_CHARACTER(" + str(len(retpars)) + "));"]+
                          outconv + sets + names +
                          ["  SET_NAMES(result, names);" ] + 
                          ["  UNPROTECT("+str(len(sets)+1)+");" ])
            
        return ret

################################################################################
# Java interface, experimental version using JNI (Java Native Interface)
# TODO: - everything :) This is just a PoC implementation.
################################################################################

class JavaCodeGenerator(CodeGenerator):
    """Class containing the common parts of JavaJavaCodeGenerator and
    JavaCCodeGenerator"""
    package = "net.sf.igraph"

    def __init__(self, func, types):
        CodeGenerator.__init__(self, func, types)

    def camelcase(s):
        """Returns a camelCase version of the given string (as used in Java
        libraries"""
        parts = s.split("_")
        result = [parts.pop(0)]
        for part in parts: result.append(part.capitalize())
        return "".join(result)
    camelcase=staticmethod(camelcase)

    def get_function_metadata(self, f, type_param="JAVATYPE"):
        """Returns metadata for the given function based on the parameters.
        f is the name of the function. The result is a dict with the following
        keys:

        - java_modifiers: Java modifiers to be used in the .java file
        - return_type: return type of the function
        - name: name of the function
        - argument_types: list of argument types
        - self_name: name of the "self" argument
        - is_static: whether the function is static
        - is_constructor: whether the function is a constructor
        """
        params = self.parse_params(f)
        is_static, is_constructor = False, False

        # We will collect data related to the current function in a dict
        data = {}
        data["name"]=self.func[f].get("NAME-JAVA", \
          JavaCodeGenerator.camelcase(f[7:]))
        data["java_modifiers"]=["public"]

        # Check parameter types to determine Java calling semantics
        types = {"IN": [], "OUT": [], "INOUT": []}
        for p in params.keys():
            types[params[p]["mode"]].append(params[p])

        if len(types["OUT"])+len(types["INOUT"]) == 1:
            # If a single one is OUT or INOUT and all others are
            # INs, then this is our lucky day - the method fits the Java
            # semantics
            if len(types["OUT"]) > 0:
                return_type_name = types["OUT"][0]["type"]
            else:
                return_type_name = types["INOUT"][0]["type"]
        elif len(types["OUT"])+len(types["INOUT"]) == 0 and \
            self.func[f].has_key("RETURN"):
            # There are only input parameters and the return type is specified,
            # this also fits the Java semantics
            return_type_name = self.func[f]["RETURN"]
        else:
            raise StimulusError, "%s: calling convention unsupported yet" % \
              data["name"]

        # Loop through the input parameters
        method_arguments = []
        found_self = False
        for p in params.keys():
            if params[p]["mode"] != "IN": continue
            type_name = params[p]["type"]
            if not found_self and type_name == "GRAPH":
                # this will be the 'self' argument
                found_self = True
                data["self_name"] = p
                continue
            tdesc = self.types.get(type_name, {})
            if not tdesc.has_key(type_param):
                raise StimulusError, "%s: unknown input type %s (needs %s), skipping" % \
                  (data["name"], type_name, type_param)
            method_arguments.append(" ".join([tdesc[type_param], p]))
        data["argument_types"] = method_arguments

        if not found_self:
            # Loop through INOUT arguments if we found no "self" yet
            for p in params.keys():
                if params[p]["mode"] == "INOUT" and params[p]["type"] == "GRAPH":
                    found_self = True
                    data["self_name"] = p
                    break

        tdesc = self.types.get(return_type_name, {})
        if not tdesc.has_key(type_param):
            raise StimulusError, "%s: unknown return type %s, skipping" % \
                (data["name"], return_type_name)
        data["return_type"] = tdesc[type_param]

        if not found_self:
            data["java_modifiers"].append("static")
            data["name"] = data["name"][0].upper()+data["name"][1:]

        data["java_modifiers"] = " ".join(data["java_modifiers"])
        data["is_static"] = not found_self
        data["is_constructor"] = is_constructor

        return data


class JavaJavaCodeGenerator(JavaCodeGenerator):
    def __init__(self, func, types):
        JavaCodeGenerator.__init__(self, func, types)

    def generate(self, inputs, output):
        out=open(output, "w")

        if len(inputs)>1:
            raise StimulusError, "Java code generator supports only a single input"

        input = open(inputs[0])
        for line in input:
            if "%STIMULUS%" not in line:
                out.write(line)
                continue

            for f in self.func.keys():
                if (self.ignore(f)): continue
                try:
                    func_metadata = self.get_function_metadata(f)
                    func_metadata["arguments"] = ", ".join(func_metadata["argument_types"])
                    out.write("    %(java_modifiers)s native %(return_type)s %(name)s(%(arguments)s);\n" % func_metadata)
                except StimulusError, e:
                    out.write("    // %s\n" % str(e))

        out.close()
      

class JavaCCodeGenerator(JavaCodeGenerator):
    def __init__(self, func, types):
        JavaCodeGenerator.__init__(self, func, types)

    def generate_function(self, function, out):
        # Ignore?
        if self.ignore(function): return

        try:
            self.metadata=self.get_function_metadata(function, "CTYPE")
        except StimulusError, e:
            out.write("/* %s */\n" % str(e))
            return

        params=self.parse_params(function)
        self.deps = self.parse_deps(function)

        # Check types
        for p in params.keys():
            tname=params[p]['type']
            if not tname in self.types.keys():
                print "W: Unknown type encountered:", tname
                return
            params[p].setdefault('mode', 'IN')

        ## Compile the output
        ## This code generator is quite difficult, so we use different
        ## functions to generate the approprite chunks and then
        ## compile them together using a simple template.
        ## See the documentation of each chunk below.
        try:
            res={}
            res['func']=function
            res['header']=self.chunk_header(function, params)
            res['decl']=self.chunk_declaration(function, params)
            res['before']=self.chunk_before(function, params)
            res['inconv']=self.chunk_inconv(function, params)
            res['call']=self.chunk_call(function, params)
            res['outconv']=self.chunk_outconv(function, params)
            res['after']=self.chunk_after(function, params)
        except StimulusError, e:
            out.write("/* %s */\n" % str(e))
            return

        # Replace into the template
        text="""
/*-------------------------------------------/
/ %(func)-42s /
/-------------------------------------------*/
%(header)s {
                                        /* Declarations */
%(decl)s

%(before)s
                                        /* Convert input */
%(inconv)s
                                        /* Call igraph */
%(call)s
                                        /* Convert output */
%(outconv)s

%(after)s

  return result;
}\n""" % res

        out.write(text)
        
    def chunk_header(self, function, params):
        """The header.
        
        The name of the function is the igraph function name minus the
        igraph_ prefix, camelcased and prefixed with the underscored
        Java classname: net_sf_igraph_Graph_. The arguments
        are mapped from the JAVATYPE key of the type dict. Static
        methods also need a 'jclass cls' argument, ordinary methods
        need 'jobject jobj'. Besides that, the Java environment pointer
        is also passed.
        """
        data = self.get_function_metadata(function, "JAVATYPE")
        types = []

        data["funcname"] = "Java_%s_Graph_%s" % \
          (self.package.replace(".", "_"), data["name"])

        if data["is_static"]:
            data["argument_types"].insert(0, "jclass cls")
        else:
            data["argument_types"].insert(0, "jobject "+data["self_name"])
        data["argument_types"].insert(0, "JNIEnv *env")

        data["types"] = ", ".join(data["argument_types"])

        res="JNIEXPORT %(return_type)s JNICALL %(funcname)s(%(types)s)" % data
        return res

    def chunk_declaration(self, function, params):
        """The declaration part of the function body
        
        There are a couple of things to declare. First a C type is
        needed for every argument, these will be supplied in the C
        igraph call. Then, all 'OUT' arguments need an appropriate variable as
        well, the result will be stored here. The return type
        of the C function also needs to be declared, that comes
        next. The result variable will contain the final result. Finally,
        if the method is not static but we are returning a new Graph object
        (e.g. in the case of igraph_linegraph), we need a jclass variable
        to store the Java class object."""
        def do_cpar(pname):
            cname="c_"+pname
            t=self.types[params[pname]['type']]
            if 'CDECL' in t:
                decl="  " + t['CDECL']
            elif 'CTYPE' in t:
                decl="  " + t['CTYPE'] + " " + cname + ";"
            else:
                decl=""            
            return decl.replace("%C%", cname).replace("%I%", pname)
        def do_jpar(pname):
            jname="j_"+pname
            t=self.types[params[pname]['type']]
            if 'JAVADECL' in t:
                decl="  " + t['JAVADECL']
            elif 'JAVATYPE' in t:
                decl="  " + t['JAVATYPE'] + " " + jname + ";"
            else:
                decl=""            
            return decl.replace("%J%", jname).replace("%I%", pname)

        inout=[ do_cpar(n) for n in params.keys() ]
        out=[ do_jpar(n) for n,p in params.items() if p['mode']=='OUT']

        rt=self.types[self.func[function]['RETURN']]    
        if 'CDECL' in rt:
            retdecl="  " + rt['CDECL']
        elif 'CTYPE' in rt:
            retdecl="  " + rt['CTYPE'] + " c__result;"
        else:
            retdecl=""

        rnames = [n for n,p in params.items() if p['mode'] in ['OUT','INOUT']]
        jretdecl = ""
        if len(rnames)>0:
            n = rnames[0]
            rtname = params[n]['type']
        else:
            rtname = self.func[function]["RETURN"]
        rt = self.types[rtname]
        if 'JAVADECL' in rt:
            jretdecl="  " + rt['JAVADECL']
        elif 'JAVATYPE' in rt:
            jretdecl="  " + rt['JAVATYPE'] + " result;"

        decls = inout + out + [retdecl, jretdecl]
        if not self.metadata["is_static"] and rtname == "GRAPH":
            self.metadata["need_class_decl"] = True
            decls.append("  jclass cls = (*env)->GetObjectClass(env, %s);" % self.metadata["self_name"])
        else:
            self.metadata["need_class_decl"] = False
        return "\n".join([i for i in decls if i!=""])

    def chunk_before(self, function, params):
        """We simply call Java_igraph_before"""
        return '  Java_igraph_before();'        

    def chunk_inconv(self, function, params):
        """Input conversions. Not only for types with mode 'IN' and
        'INOUT', eg. for 'OUT' vector types we need to allocate the
        required memory here, do all the initializations, etc. Types
        without INCONV fields are ignored. The usual %C%, %I% is
        performed at the end.
        """
        def do_par(pname):
            cname="c_"+pname
            t=self.types[params[pname]['type']]
            mode=params[pname]['mode']
            if 'INCONV' in t and mode in t['INCONV']:
                inconv="  " + t['INCONV'][mode]
            else:
                inconv=""

            if pname in self.deps.keys():
                deps = self.deps[pname]
                for i in range(len(deps)):
                    inconv=inconv.replace("%C"+str(i+1)+"%", "c_"+deps[i])
                
            return inconv.replace("%C%", cname).replace("%I%", pname)

        inconv=[ do_par(n) for n in params.keys() ]
        inconv=[ i for i in inconv if i != "" ]

        return "\n".join(inconv)

    def chunk_call(self, function, params):
        """Every single argument is included, independently of their
        mode. If a type has a 'CALL' field then that is used after the
        usual %C% and %I% substitutions, otherwise the standard 'c_'
        prefixed C argument name is used.
        """
        types=[ self.types[params[n]['type']] for n in params.keys() ]
        call=map( lambda t, n: t.get('CALL', "c_"+n), types, params.keys() )
        call=map( lambda c, n: c.replace("%C%", "c_"+n).replace("%I%", n),
                  call, params.keys() )
        lines = ["  if ((*env)->ExceptionCheck(env)) {", \
                 "    c__result = IGRAPH_EINVAL;", \
                 "  } else {", \
                 "    c__result = " + function + "(" + ", ".join(call) + ");", \
                 "  }"]
        return "\n".join(lines)

    def chunk_outconv(self, function, params):
        """The output conversions, this is quite difficult. A function
        may report its results in two ways: by returning it directly
        or by setting a variable to which a pointer was passed. igraph
        usually uses the latter and returns error codes, except for
        some simple functions like 'igraph_vcount()' which cannot
        fail.

        First we add the output conversion for all types. This is
        easy. Note that even 'IN' arguments may have output
        conversion, eg. this is the place to free memory allocated to
        them in the 'INCONV' part.

        Then we check how many 'OUT' or 'INOUT' arguments we
        have. There are three cases. If there is a single such
        argument then that is already converted and we need to return
        that. If there is no such argument then the output of the
        function was returned, so we perform the output conversion for
        the returned type and this will be the result. The case of
        more than one 'OUT' and 'INOUT' arguments is not yet supported by
        the Java interface.
        """
        def do_par(pname):
            cname="c_"+pname
            jname="j_"+pname
            t=self.types[params[pname]['type']]
            mode=params[pname]['mode']
            if 'OUTCONV' in t and mode in t['OUTCONV']:
                outconv="  " + t['OUTCONV'][mode]
            else:
                outconv=""
            return outconv.replace("%C%", cname).replace("%I%", jname)

        outconv=[ do_par(n) for n in params.keys() ]
        outconv=[ o for o in outconv if o != "" ]

        retpars=[ (n,p) for n,p in params.items() if p['mode'] in ['OUT', 'INOUT'] ]
        if len(retpars)==0:
            # return the return value of the function
            rt=self.types[self.func[function]['RETURN']]
            if 'OUTCONV' in rt:
                retconv="  " + rt['OUTCONV']['OUT']
            else:
                retconv=""
            retconv=retconv.replace("%C%", "c__result").replace("%I%", "result")
            if len(retconv)>0: outconv.append(retconv)
            ret="\n".join(outconv)
        elif len(retpars)==1:
            # return the single output value
            if retpars[0][1]['mode'] == "OUT":
                # OUT parameter
                retconv="  result = j_" + retpars[0][0] + ";"
            else:
                # INOUT parameter
                retconv="  result = " + retpars[0][0] + ";"
            outconv.append(retconv)

            outconv.insert(0, "if (c__result == 0) {")
            outconv.extend(["} else {", "  result = 0;", "}"])
            outconv = ["  %s" % line for line in outconv]
            ret="\n".join(outconv)
        else:
            raise StimulusError, "%s: the case of multiple outputs not supported yet" % function
        
        return ret

    def chunk_after(self, function, params):
        """We simply call Java_igraph_after"""
        return '  Java_igraph_after();'        



################################################################################
# Shell interface, igraph functions directly from the command line
# TODO: - read/write default input/output from/to stdin/stdout
#       - short options
#       - prefixed output (?)
#       - default values depending on other parameters
#       - other input/output graph formats, to be controlled by
#         environment variables (?): IGRAPH_INGRAPH, IGRAPH_OUTGRAPH
################################################################################

class ShellLnCodeGenerator(CodeGenerator):
    def __init__(self, func, types):
        CodeGenerator.__init__(self, func, types)

    def generate(self, inputs, output):
        out=open(output, "w")
        self.append_inputs(inputs, out)
        for f in self.func.keys():
            if (self.ignore(f)):
                continue
            out.write(f+"\n")
        out.close()

class ShellCodeGenerator(CodeGenerator):
    def __init__(self, func, types):
        CodeGenerator.__init__(self, func, types)

    def generate(self, inputs, output):
        out=open(output, "w")
        self.append_inputs(inputs, out)
        out.write("\n/* Function prototypes first */\n\n")
        
        for f in self.func.keys():
            if self.ignore(f): continue
            if 'FLAGS' in self.func[f]:
                flags=self.func[f]['FLAGS']
                flags=flags.split(",")
                flags=[ flag.strip() for flag in flags ]
            else:
                self.func[f]['FLAGS']=[]
            self.generate_prototype(f, out)

        out.write("\n/* The main function */\n\n")
        out.write("int main(int argc, char **argv) {\n\n")
        out.write("  const char *base=basename(argv[0]);\n\n  ")
        for f in self.func.keys():
            if self.ignore(f): continue
            out.write("if (!strcasecmp(base, \""+f+
                      "\")) {\n    return shell_"+f+"(argc, argv);\n  } else ")
        out.write("{\n    printf(\"Unknown function, exiting\\n\");\n")
        out.write("  }\n\n  shell_igraph_usage(argc, argv);\n  return 0;\n\n}\n");

        out.write("\n/* The functions themselves at last */\n")
        for f in self.func.keys():
            if self.ignore(f): continue
            self.generate_function(f, out)
        
        out.close()

    def generate_prototype(self, function, out):
        out.write("int shell_"+function+"(int argc, char **argv);\n")

    def generate_function(self, function, out):
        params=self.parse_params(function)

        # Check types, also enumerate them
        args=seqdict.seqdict()
        for p in params.keys():
            tname=params[p]['type']
            if not tname in self.types.keys():
                print "Error: Unknown type encountered:", tname
                sys.exit(7)
            params[p].setdefault('mode', 'IN')
            t=self.types[tname]
            mode=params[p]['mode']
            if 'INCONV' in t or 'OUTCONV' in t:
                args[p]=params[p].copy()
                args[p]['shell_no']=len(args)-1
                if mode=="INOUT":
                    args[p]['mode']='IN'
                    args[p+'-out']=params[p].copy()
                    args[p+'-out']['mode']='OUT'
                    args[p+'-out']['shell_no']=len(args)-1
                    if 'INCONV' not in t or 'IN' not in t['INCONV']:
                        print "Warning: no INCONV for type", tname, ", mode IN"
                    if 'OUTCONV' not in t or 'OUT' not in t['OUTCONV']:
                        print "Warning: no OUTCONV for type", tname, ", mode OUT"
            if mode =='IN' and ('INCONV' not in t or mode not in t['INCONV']):
                print "Warning: no INCONV for type", tname, ", mode", mode
            if mode == 'OUT' and ('OUTCONV' not in t or mode not in t['OUTCONV']):
                print "Warning: no OUTCONV for type", tname, ", mode", mode

        res={'nargs': len(args)}
        res['func']=function
        res['args']=self.chunk_args(function, args)
        res['decl']=self.chunk_decl(function, params)
        res['inconv']=self.chunk_inconv(function, args)
        res['call']=self.chunk_call(function, params)
        res['outconv']=self.chunk_outconv(function, args)
        res['default']=self.chunk_default(function, args)
        res['usage']=self.chunk_usage(function, args)
        text="""
/*-------------------------------------------/
/ %(func)-42s /
/-------------------------------------------*/
void shell_%(func)s_usage(char **argv) {
%(usage)s
  exit(1);
}

int shell_%(func)s(int argc, char **argv) {

%(decl)s

  int shell_seen[%(nargs)s];
  int shell_index=-1;
  struct option shell_options[]= { %(args)s
                                   { "help",no_argument,0,%(nargs)s },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, %(nargs)s*sizeof(int));
%(default)s
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%%s' argument given twice.\\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  
%(inconv)s
    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<%(nargs)s; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%%s'.\\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
%(call)s

  /* Write the result */
%(outconv)s

  return 0;
}\n""" % res
        out.write(text)

    def chunk_args(self, function, params):
        res=[ ['"'+n+'"',"required_argument","0", str(p['shell_no']) ]
              for n,p in params.items() ]
        res=[ "{ "+",".join(e)+" }," for e in res ]
        return "\n                                   ".join(res)

    def chunk_decl(self, function, params):
        def do_par(pname):
            t=self.types[params[pname]['type']]
            if 'DECL' in t:
                decl="  " + t['DECL'].replace("%C%", pname)
            elif 'CTYPE' in t:
                decl="  " + t['CTYPE'] + " " + pname
            else:
                decl=""
            if 'default' in params[pname]:
                if 'DEFAULT' in t and params[pname]['default'] in t['DEFAULT']:
                    default="="+t['DEFAULT'][params[pname]['default']]
                else:
                    default="="+params[pname]['default']
            else:
                default=""
            if decl: return decl+default+";"
            else: return ""

        decl=[ do_par(n) for n in params.keys() ]
        inout=[ "  char* shell_arg_"+n+"=0;" for n,p in params.items()
                if p['mode'] in ['INOUT','OUT'] ]
        rt=self.types[self.func[function]['RETURN']]
        if 'DECL' in rt:
            retdecl="  " + rt['DECL']
        elif 'CTYPE' in rt:
            retdecl="  " + rt['CTYPE'] + " shell_result;"
        else:
            retdecl=""

        if self.func[function]['RETURN'] != 'ERROR':
            retchar="  char *shell_arg_shell_result=\"-\";"
        else:
            retchar=""
        return "\n".join(decl+inout+[retdecl, retchar])

    def chunk_default(self, function, params):
        def do_par(pname):
            t=self.types[params[pname]['type']]
            if 'default' in params[pname]:
                res="  shell_seen["+str(params[pname]['shell_no'])+"]=2;"
            else:
                res=""
            return res

        res= [ do_par(n) for n in params.keys() ]
        res= [ n for n in res if n != "" ]
        return "\n".join(res)

    def chunk_inconv(self, function, params):
        def do_par(pname):
            t=self.types[params[pname]['type']]
            mode=params[pname]['mode']
            if 'INCONV' in t and mode in t['INCONV']:
                inconv="" + t['INCONV'][mode]
            else:
                inconv=""
            if pname.endswith('-out'): pname=pname[0:-4]
            return inconv.replace("%C%", pname)

        inconv=[ "    case "+str(p['shell_no'])+": /* "+n+" */\n      "+ do_par(n)
                 for n,p in params.items() ]
        inconv=[ n+"\n      break;" for n in inconv ]
        inconv=[ "".join(n) for n in inconv ]
        text="\n    switch (shell_index) {\n"+"\n".join(inconv)+ \
             "\n    case "+str(len(inconv))+":\n      shell_"+function+"_usage(argv);\n      break;"+ \
             "\n    default:\n      break;\n    }\n"
        return text

    def chunk_call(self, function, params):
        types=[ self.types[params[n]['type']] for n in params.keys() ]
        call=map( lambda t,n: t.get('CALL', n), types, params.keys() )
        call=map( lambda c,n: c.replace("%C%", n), call, params.keys() )        
        return "  shell_result=" + function + "(" + ", ".join(call) + ");"

    def chunk_outconv(self, function, params):
        def do_par(pname):
            t=self.types[params[pname]['type']]
            mode=params[pname]['mode']
            if 'OUTCONV' in t and mode in t['OUTCONV']:
                outconv="  " + t['OUTCONV'][mode]
            else:
                outconv=""
            if pname.endswith('-out'): pname=pname[0:-4]
            return outconv.replace("%C%", pname)

        outconv=[ do_par(n) for n in params.keys() ]
        rt=self.types[self.func[function]['RETURN']]
        if 'OUTCONV' in rt and 'OUT' in rt['OUTCONV']:
            rtout="  " + rt['OUTCONV']['OUT']
        else:
            rtout=""
        outconv.append(rtout.replace("%C%", "shell_result"))
        outconv=[ o for o in outconv if o != "" ]
        return "\n".join(outconv)

    def chunk_usage(self, function, params):
        res=[ "--"+n+"=<"+n+">" for n in params.keys() ]
        return "  printf(\"%s "+" ".join(res)+"\\n\", basename(argv[0]));"
    
################################################################################
if __name__ == "__main__":
    main()




from __future__ import print_function

import unittest

from igraph import *

from .utils import is_pypy, skipIf, temporary_file


class CliqueTests(unittest.TestCase):
    def setUp(self):
        self.g=Graph.Full(6)
        self.g.delete_edges([(0, 1), (0, 2), (3, 5)])

    def testCliques(self):
        tests = {(4, -1): [[1, 2, 3, 4], [1, 2, 4, 5]],
                 (2, 2): [[0, 3], [0, 4], [0, 5],
                          [1, 2], [1, 3], [1, 4], [1, 5],
                          [2, 3], [2, 4], [2, 5], [3, 4], [4, 5]],
                 (-1, -1): [[0], [1], [2], [3], [4], [5],
                            [0, 3], [0, 4], [0, 5],
                            [1, 2], [1, 3], [1, 4], [1, 5],
                            [2, 3], [2, 4], [2, 5], [3, 4], [4, 5],
                            [0, 3, 4], [0, 4, 5],
                            [1, 2, 3], [1, 2, 4], [1, 2, 5],
                            [1, 3, 4], [1, 4, 5], [2, 3, 4], [2, 4, 5],
                            [1, 2, 3, 4], [1, 2, 4, 5]]}
        for (lo, hi), exp in tests.items():
            self.assertEqual(sorted(exp), sorted(map(sorted, self.g.cliques(lo, hi))))

    def testLargestCliques(self):
        self.assertEqual(sorted(map(sorted, self.g.largest_cliques())),
                         [[1, 2, 3, 4], [1, 2, 4, 5]])

    def testMaximalCliques(self):
        self.assertEqual(sorted(map(sorted, self.g.maximal_cliques())),
                         [[0, 3, 4], [0, 4, 5],
                          [1, 2, 3, 4], [1, 2, 4, 5]])
        self.assertEqual(sorted(map(sorted, self.g.maximal_cliques(min=4))),
                         [[1, 2, 3, 4], [1, 2, 4, 5]])
        self.assertEqual(sorted(map(sorted, self.g.maximal_cliques(max=3))),
                         [[0, 3, 4], [0, 4, 5]])

    def testMaximalCliquesFile(self):
        def read_cliques(fname):
            with open(fname) as fp:
                return sorted(sorted(int(item) for item in line.split())
                                for line in fp)

        with temporary_file() as fname:
            self.g.maximal_cliques(file=fname)
            self.assertEqual([[0, 3, 4], [0, 4, 5], [1, 2, 3, 4], [1, 2, 4, 5]],
                             read_cliques(fname))

        with temporary_file() as fname:
            self.g.maximal_cliques(min=4, file=fname)
            self.assertEqual([[1, 2, 3, 4], [1, 2, 4, 5]], read_cliques(fname))

        with temporary_file() as fname:
            self.g.maximal_cliques(max=3, file=fname)
            self.assertEqual([[0, 3, 4], [0, 4, 5]], read_cliques(fname))

    def testCliqueNumber(self):
        self.assertEqual(self.g.clique_number(), 4)
        self.assertEqual(self.g.omega(), 4)


class IndependentVertexSetTests(unittest.TestCase):
    def setUp(self):
        self.g1=Graph.Tree(5, 2, TREE_UNDIRECTED)
        self.g2=Graph.Tree(10, 2, TREE_UNDIRECTED)

    def testIndependentVertexSets(self):
        tests = {(4, -1): [],
                 (2, 2): [(0, 3), (0, 4), (1, 2), (2, 3), (2, 4), (3, 4)],
                 (-1, -1): [(0,), (1,), (2,), (3,), (4,),
                            (0, 3), (0, 4), (1, 2), (2, 3), (2, 4),
                            (3, 4), (0, 3, 4), (2, 3, 4)]}
        for (lo, hi), exp in tests.items():
            self.assertEqual(exp, self.g1.independent_vertex_sets(lo, hi))

    def testLargestIndependentVertexSets(self):
        self.assertEqual(self.g1.largest_independent_vertex_sets(),
                         [(0, 3, 4), (2, 3, 4)])

    def testMaximalIndependentVertexSets(self):
        self.assertEqual(self.g2.maximal_independent_vertex_sets(),
                         [(0, 3, 4, 5, 6), (0, 3, 5, 6, 9),
                          (0, 4, 5, 6, 7, 8), (0, 5, 6, 7, 8, 9),
                          (1, 2, 7, 8, 9), (1, 5, 6, 7, 8, 9),
                          (2, 3, 4), (2, 3, 9), (2, 4, 7, 8)])

    def testIndependenceNumber(self):
        self.assertEqual(self.g2.independence_number(), 6)
        self.assertEqual(self.g2.alpha(), 6)


class MotifTests(unittest.TestCase):
    def setUp(self):
        self.g = Graph.Erdos_Renyi(100, 0.2, directed=True)

    def testDyads(self):
        """
        @note: this test is not exhaustive, it only checks whether the
          L{DyadCensus} objects "understand" attribute and item accessors
        """
        dc = self.g.dyad_census()
        accessors = ["mut", "mutual", "asym", "asymm", "asymmetric", "null"]
        for a in accessors:
            self.assertTrue(isinstance(getattr(dc, a), int))
            self.assertTrue(isinstance(dc[a], int))
        self.assertTrue(isinstance(list(dc), list))
        self.assertTrue(isinstance(tuple(dc), tuple))
        self.assertTrue(len(list(dc)) == 3)
        self.assertTrue(len(tuple(dc)) == 3)

    def testTriads(self):
        """
        @note: this test is not exhaustive, it only checks whether the
          L{TriadCensus} objects "understand" attribute and item accessors
        """
        tc = self.g.triad_census()
        accessors = ["003", "012", "021d", "030C"]
        for a in accessors:
            self.assertTrue(isinstance(getattr(tc, "t"+a), int))
            self.assertTrue(isinstance(tc[a], int))
        self.assertTrue(isinstance(list(tc), list))
        self.assertTrue(isinstance(tuple(tc), tuple))
        self.assertTrue(len(list(tc)) == 16)
        self.assertTrue(len(tuple(tc)) == 16)

class CliqueBenchmark(object):
    """This is a benchmark, not a real test case. You can run it
    using:

    >>> from igraph.test.cliques import CliqueBenchmark
    >>> CliqueBenchmark().run()
    """

    def __init__(self):
        from time import time
        import gc
        self.time = time
        self.gc_collect = gc.collect

    def run(self):
        self.printIntro()
        self.testRandom()
        self.testMoonMoser()
        self.testGRG()

    def printIntro(self):
        print("n = number of vertices")
        print("#cliques = number of maximal cliques found")
        print("t1 = time required to determine the clique number")
        print("t2 = time required to determine and save all maximal cliques")
        print()

    def timeit(self, g):
        start = self.time()
        omega = g.clique_number()
        mid = self.time()
        cl = g.maximal_cliques()
        end = self.time()
        self.gc_collect()
        return len(cl), mid-start, end-mid

    def testRandom(self):
        np = {100: [0.6, 0.7],
              300: [0.1, 0.2, 0.3, 0.4],
              500: [0.1, 0.2, 0.3],
              700: [0.1, 0.2],
              1000:[0.1, 0.2],
              10000: [0.001, 0.003, 0.005, 0.01, 0.02]}

        print()
        print("Erdos-Renyi random graphs")
        print("       n        p #cliques        t1        t2")
        for n in sorted(np.keys()):
            for p in np[n]:
                g = Graph.Erdos_Renyi(n, p)
                result = self.timeit(g)
                print("%8d %8.3f %8d %8.4fs %8.4fs" % tuple([n, p] + list(result)))

    def testMoonMoser(self):
        ns = [15, 27, 33]

        print()
        print("Moon-Moser graphs")
        print("       n exp_clqs #cliques        t1        t2")
        for n in ns:
            n3 = n/3
            types = range(n3) * 3
            el = [(i, j) for i in range(n) for j in range(i+1,n) if types[i] != types[j]]
            g = Graph(n, el)
            result = self.timeit(g)
            print("%8d %8d %8d %8.4fs %8.4fs" % tuple([n, (3**(n/3))] + list(result)))

    def testGRG(self):
        ns = [100, 1000, 5000, 10000, 25000, 50000]

        print()
        print("Geometric random graphs")
        print("       n        d #cliques        t1        t2")
        for n in ns:
            d = 2. / (n ** 0.5)
            g = Graph.GRG(n, d)
            result = self.timeit(g)
            print("%8d %8.3f %8d %8.4fs %8.4fs" % tuple([n, d] + list(result)))


def suite():
    clique_suite = unittest.makeSuite(CliqueTests)
    indvset_suite = unittest.makeSuite(IndependentVertexSetTests)
    motif_suite = unittest.makeSuite(MotifTests)
    return unittest.TestSuite([clique_suite, indvset_suite, motif_suite])

def test():
    runner = unittest.TextTestRunner()
    runner.run(suite())

if __name__ == "__main__":
    test()




import unittest
from igraph import Graph, Layout, BoundingBox


class LayoutTests(unittest.TestCase):
    def testConstructor(self):
        layout = Layout([(0,0,1), (0,1,0), (1,0,0)])
        self.assertEqual(layout.dim, 3)
        layout = Layout([(0,0,1), (0,1,0), (1,0,0)], 3)
        self.assertEqual(layout.dim, 3)
        self.assertRaises(ValueError, Layout, [(0,1), (1,0)], 3)

    def testIndexing(self):
        layout = Layout([(0,0,1), (0,1,0), (1,0,0), (2,1,3)])
        self.assertEqual(len(layout), 4)
        self.assertEqual(layout[1], [0, 1, 0])
        self.assertEqual(layout[3], [2, 1, 3])

        row = layout[2]
        row[2] = 1
        self.assertEqual(layout[2], [1, 0, 1])

        del layout[1]
        self.assertEqual(len(layout), 3)

    def testScaling(self):
        layout = Layout([(0,0,1), (0,1,0), (1,0,0), (2,1,3)])
        layout.scale(1.5)
        self.assertEqual(layout.coords, [[0., 0., 1.5], \
                                         [0., 1.5, 0.], \
                                         [1.5, 0., 0.], \
                                         [3., 1.5, 4.5]])

        layout = Layout([(0,0,1), (0,1,0), (1,0,0), (2,1,3)])
        layout.scale(1, 1, 3)
        self.assertEqual(layout.coords, [[0, 0, 3], \
                                         [0, 1, 0], \
                                         [1, 0, 0], \
                                         [2, 1, 9]])

        layout = Layout([(0,0,1), (0,1,0), (1,0,0), (2,1,3)])
        layout.scale((2, 2, 1))
        self.assertEqual(layout.coords, [[0, 0, 1], \
                                         [0, 2, 0], \
                                         [2, 0, 0], \
                                         [4, 2, 3]])

        self.assertRaises(ValueError, layout.scale, 2, 3)

    def testTranslation(self):
        layout = Layout([(0,0,1), (0,1,0), (1,0,0), (2,1,3)])
        layout2 = layout.copy()

        layout.translate(1,3,2)
        self.assertEqual(layout.coords, [[1, 3, 3], \
                                         [1, 4, 2], \
                                         [2, 3, 2], \
                                         [3, 4, 5]])

        layout.translate((-1,-3,-2))
        self.assertEqual(layout.coords, layout2.coords)

        self.assertRaises(ValueError, layout.translate, v=[3])

    def testCentroid(self):
        layout = Layout([(0,0,1), (0,1,0), (1,0,0), (2,1,3)])
        centroid = layout.centroid()
        self.assertEqual(len(centroid), 3)
        self.assertAlmostEqual(centroid[0], 0.75)
        self.assertAlmostEqual(centroid[1], 0.5)
        self.assertAlmostEqual(centroid[2], 1.)

    def testBoundaries(self):
        layout = Layout([(0,0,1), (0,1,0), (1,0,0), (2,1,3)])
        self.assertEqual(layout.boundaries(), ([0,0,0],[2,1,3]))
        self.assertEqual(layout.boundaries(1), ([-1,-1,-1],[3,2,4]))

        layout = Layout([])
        self.assertRaises(ValueError, layout.boundaries)
        layout = Layout([], dim=3)
        self.assertRaises(ValueError, layout.boundaries)

    def testBoundingBox(self):
        layout = Layout([(0,1), (2,7)])
        self.assertEqual(layout.bounding_box(), BoundingBox(0,1,2,7))
        self.assertEqual(layout.bounding_box(1), BoundingBox(-1,0,3,8))
        layout = Layout([])
        self.assertEqual(layout.bounding_box(), BoundingBox(0,0,0,0))

    def testCenter(self):
        layout = Layout([(-2,0), (-2,-2), (0,-2), (0,0)])
        layout.center()
        self.assertEqual(layout.coords, [[-1,1], [-1,-1], [1,-1], [1,1]])
        layout.center(5,5)
        self.assertEqual(layout.coords, [[4,6], [4,4], [6,4], [6,6]])
        self.assertRaises(ValueError, layout.center, 3)
        self.assertRaises(TypeError, layout.center, p=6)

    def testFitInto(self):
        layout = Layout([(-2,0), (-2,-2), (0,-2), (0,0)])
        layout.fit_into(BoundingBox(5,5,8,10), keep_aspect_ratio=False)
        self.assertEqual(layout.coords, [[5, 10], [5, 5], [8, 5], [8, 10]])
        layout = Layout([(-2,0), (-2,-2), (0,-2), (0,0)])
        layout.fit_into(BoundingBox(5,5,8,10))
        self.assertEqual(layout.coords, [[5, 9], [5, 6], [8, 6], [8, 9]])

        layout = Layout([(-1,-1,-1), (0,0,0), (1,1,1), (2,2,0), (3,3,-1)])
        layout.fit_into((0,0,0,8,8,4))
        self.assertEqual(layout.coords, \
                [[0, 0, 0], [2, 2, 2], [4, 4, 4], [6, 6, 2], [8, 8, 0]]
        )

        layout = Layout([])
        layout.fit_into((6,7,8,11))
        self.assertEqual(layout.coords, [])

    def testToPolar(self):
        layout = Layout([(0, 0), (-1, 1), (0, 1), (1, 1)])
        layout.to_radial(min_angle=180, max_angle=0, max_radius=2)
        exp = [[0., 0.], [-2., 0.], [0., 2.], [2, 0.]]
        for idx in range(4):
            self.assertAlmostEqual(layout.coords[idx][0], exp[idx][0], places=3)
            self.assertAlmostEqual(layout.coords[idx][1], exp[idx][1], places=3)

    def testTransform(self):
        def tr(coord, dx, dy): return coord[0]+dx, coord[1]+dy
        layout = Layout([(1, 2), (3, 4)])
        layout.transform(tr, 2, -1)
        self.assertEqual(layout.coords, [[3, 1], [5, 3]])


class LayoutAlgorithmTests(unittest.TestCase):
    def testAuto(self):
        def layout_test(graph, test_with_dims=(2, 3)):
            lo = graph.layout("auto")
            self.assertTrue(isinstance(lo, Layout))
            self.assertEqual(len(lo[0]), 2)
            for dim in test_with_dims:
                lo = graph.layout("auto", dim=dim)
                self.assertTrue(isinstance(lo, Layout))
                self.assertEqual(len(lo[0]), dim)
            return lo

        g = Graph.Barabasi(10)
        layout_test(g)

        g = Graph.GRG(101, 0.2)
        del g.vs["x"]
        del g.vs["y"]
        layout_test(g)

        g = Graph.Full(10) * 2
        layout_test(g)

        g["layout"] = "graphopt"
        layout_test(g, test_with_dims=())

        g.vs["x"] = range(20)
        g.vs["y"] = range(20, 40)
        layout_test(g, test_with_dims=())

        del g["layout"]
        lo = layout_test(g, test_with_dims=(2,))
        self.assertEqual([tuple(item) for item in lo],
                         list(zip(range(20), range(20, 40))))

        g.vs["z"] = range(40, 60)
        lo = layout_test(g)
        self.assertEqual([tuple(item) for item in lo],
                         list(zip(range(20), range(20, 40), range(40, 60))))

    def testCircle(self):
        def test_is_proper_circular_layout(graph, layout):
            xs, ys = zip(*layout)
            n = graph.vcount()
            self.assertEqual(n, len(xs))
            self.assertEqual(n, len(ys))
            self.assertAlmostEqual(0, sum(xs))
            self.assertAlmostEqual(0, sum(ys))
            for x, y in zip(xs, ys):
                self.assertAlmostEqual(1, x**2+y**2)

        g = Graph.Ring(8)
        layout = g.layout("circle")
        test_is_proper_circular_layout(g, g.layout("circle"))

        order = [0, 2, 4, 6, 1, 3, 5, 7]
        ordered_layout = g.layout("circle", order=order)
        test_is_proper_circular_layout(g, g.layout("circle"))
        for v, w in enumerate(order):
            self.assertAlmostEqual(layout[v][0], ordered_layout[w][0])
            self.assertAlmostEqual(layout[v][1], ordered_layout[w][1])

    def testDavidsonHarel(self):
        # Quick smoke testing only
        g = Graph.Barabasi(100)
        lo = g.layout("dh")
        self.assertTrue(isinstance(lo, Layout))

    def testFruchtermanReingold(self):
        g = Graph.Barabasi(100)

        lo = g.layout("fr")
        self.assertTrue(isinstance(lo, Layout))

        lo = g.layout("fr", miny=range(100))
        self.assertTrue(isinstance(lo, Layout))
        self.assertTrue(all(lo[i][1] >= i for i in range(100)))

        lo = g.layout("fr", miny=range(100), maxy=range(100))
        self.assertTrue(isinstance(lo, Layout))
        self.assertTrue(all(lo[i][1] == i for i in range(100)))

        lo = g.layout("fr", miny=[2]*100, maxy=[3]*100, minx=[4]*100, maxx=[6]*100)
        self.assertTrue(isinstance(lo, Layout))
        bbox = lo.bounding_box()
        self.assertTrue(bbox.top >= 2)
        self.assertTrue(bbox.bottom <= 3)
        self.assertTrue(bbox.left >= 4)
        self.assertTrue(bbox.right <= 6)

    def testFruchtermanReingoldGrid(self):
        g = Graph.Barabasi(100)
        for grid_opt in ["grid", "nogrid", "auto", True, False]:
            lo = g.layout("fr", miny=range(100), grid=grid_opt)
            self.assertTrue(isinstance(lo, Layout))
            self.assertTrue(all(lo[i][1] >= i for i in range(100)))

    def testKamadaKawai(self):
        g = Graph.Barabasi(100)
        lo = g.layout("kk", miny=[2]*100, maxy=[3]*100, minx=[4]*100, maxx=[6]*100)
        self.assertTrue(isinstance(lo, Layout))
        bbox = lo.bounding_box()
        self.assertTrue(bbox.top >= 2)
        self.assertTrue(bbox.bottom <= 3)
        self.assertTrue(bbox.left >= 4)
        self.assertTrue(bbox.right <= 6)

    def testMDS(self):
        g = Graph.Tree(10, 2)
        lo = g.layout("mds")
        self.assertTrue(isinstance(lo, Layout))

        dists = g.shortest_paths()
        lo = g.layout("mds", dists)
        self.assertTrue(isinstance(lo, Layout))

        g += Graph.Tree(10, 2)
        lo = g.layout("mds")
        self.assertTrue(isinstance(lo, Layout))

    def testReingoldTilford(self):
        g = Graph.Barabasi(100)
        lo = g.layout("rt")
        ys = [coord[1] for coord in lo]
        root = ys.index(0.0)
        self.assertEqual(ys, g.shortest_paths(root)[0])
        g = Graph.Barabasi(100) + Graph.Barabasi(50)
        lo = g.layout("rt", root=[0, 100])
        self.assertEqual(lo[100][1]-lo[0][1], 0)
        lo = g.layout("rt", root=[0, 100], rootlevel=[2, 10])
        self.assertEqual(lo[100][1]-lo[0][1], 8)

    def testBipartite(self):
        g = Graph.Full_Bipartite(3, 2)

        lo = g.layout("bipartite")
        ys = [coord[1] for coord in lo]
        self.assertEqual([1, 1, 1, 0, 0], ys)

        lo = g.layout("bipartite", vgap=3)
        ys = [coord[1] for coord in lo]
        self.assertEqual([3, 3, 3, 0, 0], ys)

        lo = g.layout("bipartite", hgap=5)
        self.assertEqual(set([0, 5, 10]), set(coord[0] for coord in lo if coord[1] == 1))
        self.assertEqual(set([2.5, 7.5]), set(coord[0] for coord in lo if coord[1] == 0))

    def testDRL(self):
        # Regression test for bug #1091891
        g = Graph.Ring(10, circular=False) + 1
        lo = g.layout("drl")
        self.assertTrue(isinstance(lo, Layout))


def suite():
    layout_suite = unittest.makeSuite(LayoutTests)
    layout_algorithm_suite = unittest.makeSuite(LayoutAlgorithmTests)
    return unittest.TestSuite([layout_suite, layout_algorithm_suite])


def test():
    runner = unittest.TextTestRunner()
    runner.run(suite())


if __name__ == "__main__":
    test()



################################################################################
#  Sequential Dictionary Class                                                 #
#                                                                              #
#  by Wolfgang Grafen                                                          #
#                                                                              #
#  Version 0.2    11. February 2004
#                                                                              #
#  email to: WolfgangGrafen@gmx.de                                             #
#                                                                              #
################################################################################

from ndict import seqdict   #Sequential Single Value Dictionary
from UserList import UserList

class MyUserList(UserList):
    from UserList import UserList
    def __init__(self,parent,liste=None):
      UserList.__init__(self,liste)
      self.parent = parent  #remember parent for call-back
    def __delitem__(self, i):
      del self.data[i]
      if self.data==[]:     #call-back, deletes item of parent
        index = self.parent.values().index([])
        del self.parent[index:index+1]
        
class mseqdict(seqdict):    #Sequential Multiple Value Dictionary
  def __init__(self,List=[],Dict={}):
    self.list = []
    self.dict = {}
    if not List:
      pass     
    elif type(List)==type({}):
      for key,value in List.items():
        self.__setitem__(key,value)
    elif List and not Dict: #dict.items()
      for key,value in List:
        if isinstance(value,MyUserList):
          for v in value:
            self.__setitem__(key,v)
        else:
          self.__setitem__(key,value)     
    elif type(List)==type(Dict)==type([]):
      for key,value in map(None,List,Dict):
        self.__setitem__(key,value)
    else:
      if isinstance(Dict.values()[0],MyUserList):
        self.dict = Dict
        self.list = List
      else:
        for key in List:
          value = Dict[key]
          if type(value)==type([]):
            for v in value:
              self.__setitem__(key,v)
          else:
            self.__setitem__(key,value)     

    self_list = self.list
    self_dict = self.dict
    for k in self_list:
        assert self_dict.has_key(k),"key %r not in self.dict" % k

    for k in self_dict.keys():
        if k not in self_list:
            self_list.append(k)

  def __setitem__(self,key,value):
    if not self.dict.has_key(key):
      self.list.append(key)
      if isinstance(value,MyUserList):
        self.dict[key] = value
      else:
        self.dict[key]=MyUserList(self,[value])
    else:
      values = self.dict[key]
      if isinstance(value,MyUserList):
        for v in value:
          if not v in values:
            values.extend(MyUserList(self,[v]))
      else:
        #if not value in values:
        for v in values:
            if v is value:
                break
            values.extend(MyUserList(self,[value]))
  def __delitem__(self, key):
    del self.dict[key]
    self.list.remove(key)      

  def append(self,key,value):
    self.__setitem__(key,value)                 
  def __setslice__(self,start,stop,newdict):
    start = max(start,0); stop = max(stop,0)
    delindexes = []
    for key in newdict.keys():
      if self.dict.has_key(key):
        index = self.list.index(key)
        delindexes.append(index)
        if index < start:
          start = start - 1
          stop  = stop  - 1
        elif index >= stop:
          pass
        else:
          stop  = stop - 1
      else:
        self.dict[key]=UserList(self)
    delindexes.sort()
    delindexes.reverse()
    for index in delindexes:
      key = self.list[index]
      #del   self.dict[key]
      del  self.list[index]
    self.list[start:stop] = newdict.list[:]
    self.dict.update(newdict.dict)  
  def copy(self):
    values = map(lambda x:x[:],self.values())
    return self.__class__(self.list,values)
  def count(self,value):
    vallist = self.dict.values()
    return map(lambda x,y=value:x.count(y),vallist).count(1)
  def filter(self,function,filtervalues=0):
    if   filtervalues == 1: #consider key and all keyvalues at once
      dict = self.__class__()
      for key,values in self.items():
        if function(key,values):
          dict[key]=values
      return dict
    elif filtervalues == 2: #consider key and every keyvalue for itself
      dict = self.__class__()
      for key,values in self.items():
        for value in values:
          if function(key,value):
            dict[key]=value
      return dict
    else:                   #consider key only
      liste=filter(function,self.list)
      dict = {}
      for i in liste:
        dict[i]=self.dict[i]
      return self.__class__(liste,dict)
  def map(self,function,mapvalues=2):
    if   mapvalues == 1:    #consider key and all keyvalues at once
      dict = self.__class__()
      for key,values in self.items():
        k,v = function(key,values)
        dict[k]=v
      return dict
    else: #if mapvalues!=1: #consider key and every keyvalue for itself
      dict = self.__class__()
      for key,values in self.items():
        for value in values:
          k,v = function(key,value)
          dict[k]=v
      return dict
  def pop(self,key='...None',value='...None'):
    if value=='...None':
      if key=='...None':
        pos = -1
        key = self.list[pos]
      else:
        pos = self.list.index(key)
      tmp = self.dict[key]
      del self.dict[key]
      return {self.list.pop(pos):tmp}
    else:
      val = self.dict[key]
      index = val.index(value)
      tmp = val[index]
      del val[index]
      return {key:tmp}
  def remove(self,key,value='...None'):
    if value=='...None':
      del self[key]
    else:
      index = self[key].index(value) 
      del self[key][index]
  def sort(self,func1=None,func2=None):
    if not func1:
      self.list.sort()
    else:
      apply(self.list.sort,[func1])
    if func2:
      for value in self.values():
        apply(value.sort,[func2])
      
  def swap(self):
    tmp = self.__class__()
    for key,values in self.items():
      for value in values:
        tmp[value]=key
    self.list,self.dict = tmp.list,tmp.dict
    del tmp
  
  def __repr__(self):return 'mseqdict(\n%s,\n%s)'%(self.list,self.dict)



#! /usr/bin/env python

import sys
from xml.etree.ElementTree import ElementTree

def usage():
    print sys.argv[0], "<infile> <outfile>"

def main():
    if len(sys.argv) != 3:
        usage()
        sys.exit(2)
    
    # Read in
    tree=ElementTree()
    tree.parse(sys.argv[1])
    
    # Remove examples
    examples=tree.findall(".//example")
    for ex in examples:
        prog=ex.find("programlisting")
        ex.remove(prog)

    # Write result
    tree.write(sys.argv[2])    

if __name__ == "__main__":
    main()



import unittest
from igraph import *

class DirectedUndirectedTests(unittest.TestCase):
    def testToUndirected(self):
        graph = Graph([(0,1), (0,2), (1,0)], directed=True)

        graph2 = graph.copy()
        graph2.to_undirected(mode=False)
        self.assertTrue(graph2.vcount() == graph.vcount())
        self.assertTrue(graph2.is_directed() == False)
        self.assertTrue(sorted(graph2.get_edgelist()) == [(0,1), (0,1), (0,2)])

        graph2 = graph.copy()
        graph2.to_undirected()
        self.assertTrue(graph2.vcount() == graph.vcount())
        self.assertTrue(graph2.is_directed() == False)
        self.assertTrue(sorted(graph2.get_edgelist()) == [(0,1), (0,2)])

        graph2 = graph.copy()
        graph2.es["weight"] = [1,2,3]
        graph2.to_undirected(mode="collapse", combine_edges="sum")
        self.assertTrue(graph2.vcount() == graph.vcount())
        self.assertTrue(graph2.is_directed() == False)
        self.assertTrue(sorted(graph2.get_edgelist()) == [(0,1), (0,2)])
        self.assertTrue(graph2.es["weight"] == [4,2])

        graph = Graph([(0,1),(1,0),(0,1),(1,0),(2,1),(1,2)], directed=True)
        graph2 = graph.copy()
        graph2.es["weight"] = [1,2,3,4,5,6]
        graph2.to_undirected(mode="mutual", combine_edges="sum")
        self.assertTrue(graph2.vcount() == graph.vcount())
        self.assertTrue(graph2.is_directed() == False)
        self.assertTrue(sorted(graph2.get_edgelist()) == [(0,1), (0,1), (1,2)])
        self.assertTrue(graph2.es["weight"] == [7,3,11] or graph2.es["weight"] == [3,7,11])

    def testToDirected(self):
        graph = Graph([(0,1), (0,2), (2,3), (2,4)], directed=False)
        graph.to_directed()
        self.assertTrue(graph.is_directed())
        self.assertTrue(graph.vcount() == 5)
        self.assertTrue(sorted(graph.get_edgelist()) == \
                [(0,1), (0,2), (1,0), (2,0), (2,3), (2,4), (3,2), (4,2)]
        )


class GraphRepresentationTests(unittest.TestCase):
    def testGetAdjacency(self):
        # Undirected case
        g = Graph.Tree(6, 3)
        g.es["weight"] = range(5)
        self.assertTrue(g.get_adjacency() == Matrix([
            [0, 1, 1, 1, 0, 0],
            [1, 0, 0, 0, 1, 1],
            [1, 0, 0, 0, 0, 0],
            [1, 0, 0, 0, 0, 0],
            [0, 1, 0, 0, 0, 0],
            [0, 1, 0, 0, 0, 0]
        ]))
        self.assertTrue(g.get_adjacency(attribute="weight") == Matrix([
            [0, 0, 1, 2, 0, 0],
            [0, 0, 0, 0, 3, 4],
            [1, 0, 0, 0, 0, 0],
            [2, 0, 0, 0, 0, 0],
            [0, 3, 0, 0, 0, 0],
            [0, 4, 0, 0, 0, 0]
        ]))
        self.assertTrue(g.get_adjacency(eids=True) == Matrix([
            [0, 1, 2, 3, 0, 0],
            [1, 0, 0, 0, 4, 5],
            [2, 0, 0, 0, 0, 0],
            [3, 0, 0, 0, 0, 0],
            [0, 4, 0, 0, 0, 0],
            [0, 5, 0, 0, 0, 0]
        ])-1)

        # Directed case
        g = Graph.Tree(6, 3, "tree_out")
        g.add_edges([(0,1), (1,0)])
        self.assertTrue(g.get_adjacency() == Matrix([
            [0, 2, 1, 1, 0, 0],
            [1, 0, 0, 0, 1, 1],
            [0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0]
        ]))

    def testGetSparseAdjacency(self):
        try:
            from scipy import sparse
            import numpy as np
        except ImportError:
            self.skipTest("Scipy and numpy are dependencies of this test.")

        # Undirected case
        g = Graph.Tree(6, 3)
        g.es["weight"] = range(5)
        self.assertTrue(np.all(
            (g.get_adjacency_sparse() == np.array(g.get_adjacency().data))
        ))
        self.assertTrue(np.all(
            (g.get_adjacency_sparse(attribute="weight") == np.array(g.get_adjacency(attribute="weight").data))
        ))

        # Directed case
        g = Graph.Tree(6, 3, "tree_out")
        g.add_edges([(0,1), (1,0)])
        self.assertTrue(np.all(
            g.get_adjacency_sparse() == np.array(g.get_adjacency().data)
        ))


def suite():
    direction_suite = unittest.makeSuite(DirectedUndirectedTests)
    representation_suite = unittest.makeSuite(GraphRepresentationTests)
    return unittest.TestSuite([direction_suite,
        representation_suite])

def test():
    runner = unittest.TextTestRunner()
    runner.run(suite())
    
if __name__ == "__main__":
    test()




from __future__ import division

import warnings
import unittest
from igraph import *


class AtlasTestBase(object):
    def testPageRank(self):
        for idx, g in enumerate(self.__class__.graphs):
            try:
                pr = g.pagerank()
            except Exception as ex:
                self.assertTrue(False, msg="PageRank calculation threw exception for graph #%d: %s" % (idx, ex))
                raise

            if g.vcount() == 0:
                self.assertEqual([], pr)
                continue

            self.assertAlmostEqual(1.0, sum(pr), places=5, \
                    msg="PageRank sum is not 1.0 for graph #%d (%r)" % (idx, pr))
            self.assertTrue(min(pr) >= 0, \
                    msg="Minimum PageRank is less than 0 for graph #%d (%r)" % (idx, pr))

    def testEigenvectorCentrality(self):
        # Temporarily turn off the warning handler because g.evcent() will print
        # a warning for DAGs
        warnings.simplefilter("ignore")

        try:
            for idx, g in enumerate(self.__class__.graphs):
                if idx in self.__class__.skip_graphs:
                    # Skip this graph; it causes lots of problems and I don't know why
                    continue

                try:
                    ec, eval = g.evcent(return_eigenvalue=True)
                except Exception as ex:
                    self.assertTrue(False, msg="Eigenvector centrality threw exception for graph #%d: %s" % (idx, ex))
                    raise

                if g.vcount() == 0:
                    self.assertEqual([], ec)
                    continue

                if not g.is_connected():
                    # Skip disconnected graphs; this will be fixed in igraph 0.7
                    continue

                n = g.vcount()
                if abs(eval) < 1e-4:
                    self.assertTrue(min(ec) >= -1e-10,
                            msg="Minimum eigenvector centrality is smaller than 0 for graph #%d" % idx)
                    self.assertTrue(max(ec) <= 1,
                            msg="Maximum eigenvector centrality is greater than 1 for graph #%d" % idx)
                    continue

                self.assertAlmostEqual(max(ec), 1, places=7, \
                        msg="Maximum eigenvector centrality is %r (not 1) for graph #%d (%r)" % \
                        (max(ec), idx, ec))
                self.assertTrue(min(ec) >= 0, \
                        msg="Minimum eigenvector centrality is less than 0 for graph #%d" % idx)

                ec2 = [sum(ec[u.index] for u in v.predecessors()) for v in g.vs]
                for i in range(n):
                    self.assertAlmostEqual(ec[i] * eval, ec2[i], places=7, \
                            msg="Eigenvector centrality in graph #%d seems to be invalid "\
                            "for vertex %d" % (idx, i))
        finally:
            # Reset the warning handler
            warnings.resetwarnings()

    def testHubScore(self):
        for idx, g in enumerate(self.__class__.graphs):
            sc = g.hub_score()
            if g.vcount() == 0:
                self.assertEqual([], sc)
                continue

            self.assertAlmostEqual(max(sc), 1, places=7, \
                    msg="Maximum authority score is not 1 for graph #%d" % idx)
            self.assertTrue(min(sc) >= 0, \
                    msg="Minimum hub score is less than 0 for graph #%d" % idx)

    def testAuthorityScore(self):
        for idx, g in enumerate(self.__class__.graphs):
            sc = g.authority_score()
            if g.vcount() == 0:
                self.assertEqual([], sc)
                continue

            self.assertAlmostEqual(max(sc), 1, places=7, \
                    msg="Maximum authority score is not 1 for graph #%d" % idx)
            self.assertTrue(min(sc) >= 0, \
                    msg="Minimum authority score is less than 0 for graph #%d" % idx)

class GraphAtlasTests(unittest.TestCase, AtlasTestBase):
    graphs = [Graph.Atlas(i) for i in range(1253)]
    skip_graphs = set([180])

class IsoclassTests(unittest.TestCase, AtlasTestBase):
    graphs = [Graph.Isoclass(3, i, directed=True) for i in range(16)] + \
             [Graph.Isoclass(4, i, directed=True) for i in range(218)]
    skip_graphs = set([136])

def suite():
    atlas_suite = unittest.makeSuite(GraphAtlasTests)
    isoclass_suite = unittest.makeSuite(IsoclassTests)
    return unittest.TestSuite([atlas_suite, isoclass_suite])

def test():
    runner = unittest.TextTestRunner()
    runner.run(suite())
    
if __name__ == "__main__":
    test()




# vim:ts=4:sw=4:sts=4:et
# -*- coding: utf-8 -*-
"""
Compatibility methods and backported versions of newer Python features
to enable igraph to run on Python 2.5.
"""

import sys

__license__ = u"""\
Copyright (C) 2006-2012  Tamás Nepusz <ntamas@gmail.com>
Pázmány Péter sétány 1/a, 1117 Budapest, Hungary

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc.,  51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA
"""

#############################################################################
# Simulating math.isnan

try:
    from math import isnan
except ImportError:
    def isnan(num):
        return num != num

#############################################################################
# Providing @property.setter syntax for Python 2.5

if sys.version_info < (2, 6):
    _property = property

    class property(property):
        def __init__(self, fget, *args, **kwds):
            self.__doc__ = fget.__doc__
            super(property, self).__init__(fget, *args, **kwds)

        def setter(self, fset):
            cls_ns = sys._getframe(1).f_locals
            for k, v in cls_ns.iteritems():
                if v == self:
                    propname = k
                    break
            cls_ns[propname] = property(self.fget, fset, self.fdel, self.__doc__)
            return cls_ns[propname]
else:
    if isinstance(__builtins__, dict):
        # This branch is for CPython
        property = __builtins__["property"]
    else:
        # This branch is for PyPy
        property = __builtins__.property

#############################################################################
# Providing BytesIO for Python 2.5

try:
    from io import BytesIO
except ImportError:
    # We are on Python 2.5 or earlier because Python 2.6 has a BytesIO
    # class already
    from cStringIO import StringIO
    BytesIO = StringIO



import unittest

from igraph import *

from .utils import skipIf

try:
    import numpy as np
except ImportError:
    np = None

class OperatorTests(unittest.TestCase):
    def testComplementer(self):
        g = Graph.Full(3)
        g2 = g.complementer()
        self.assertTrue(g2.vcount() == 3 and g2.ecount() == 3)
        self.assertTrue(sorted(g2.get_edgelist()) == [(0, 0), (1, 1), (2, 2)])

        g = Graph.Full(3) + Graph.Full(2)
        g2 = g.complementer(False)
        self.assertTrue(sorted(g2.get_edgelist()) == [
            (0, 3), (0, 4), (1, 3), (1, 4), (2, 3), (2, 4)
        ])

        g2 = g.complementer(loops=True)
        self.assertTrue(sorted(g2.get_edgelist()) == [
            (0, 0), (0, 3), (0, 4), (1, 1), (1, 3), (1, 4), (2, 2), (2, 3), (2, 4),
            (3, 3), (4, 4)
        ])

    def testMultiplication(self):
        g = Graph.Full(3) * 3
        self.assertTrue(g.vcount() == 9 and g.ecount() == 9
                        and g.clusters().membership == [0, 0, 0, 1, 1, 1, 2, 2, 2])

    def testDifference(self):
        g = Graph.Tree(7, 2) - Graph.Lattice([7])
        self.assertTrue(g.vcount() == 7 and g.ecount() == 5)
        self.assertTrue(sorted(g.get_edgelist()) == [(0, 2), (1, 3), (1, 4), (2, 5), (2, 6)])

    def testDifferenceWithSelfLoop(self):
        # https://github.com/igraph/igraph/issues/597#
        g = Graph.Ring(10) + [(0, 0)]
        g -= Graph.Ring(5)
        self.assertTrue(g.vcount() == 10 and g.ecount() == 7)
        self.assertTrue(sorted(g.get_edgelist()) == [(0, 0), (0, 9), (4, 5), (5, 6), (6, 7), (7, 8), (8, 9)])

    def testIntersection(self):
        g = Graph.Tree(7, 2) & Graph.Lattice([7])
        self.assertTrue(g.get_edgelist() == [(0, 1)])

    def testIntersectionMethod(self):
        g = Graph.Tree(7, 2).intersection(Graph.Lattice([7]))
        self.assertTrue(g.get_edgelist() == [(0, 1)])

    def testIntersectionNoGraphs(self):
        self.assertRaises(ValueError, intersection, [])

    def testIntersectionSingle(self):
        g1 = Graph.Tree(7, 2)
        g = intersection([g1])
        self.assertTrue(g != g1)
        self.assertTrue(g.vcount() == g1.vcount() and g.ecount() == g1.ecount())
        self.assertTrue(g.is_directed() == g1.is_directed())
        self.assertTrue(g.get_edgelist() == g1.get_edgelist())

    def testDisjointUnion(self):
        g1 = Graph.Tree(7, 2)
        g2 = Graph.Lattice([7])

        # Method
        g = g1.disjoint_union(g2)
        self.assertTrue(g.vcount() == 14 and g.ecount() == 13)

        # Module function
        g = disjoint_union([g1, g2])
        self.assertTrue(g.vcount() == 14 and g.ecount() == 13)

    def testDisjointUnionNoGraphs(self):
        self.assertRaises(ValueError, disjoint_union, [])

    def testDisjointUnionSingle(self):
        g1 = Graph.Tree(7, 2)
        g = disjoint_union([g1])
        self.assertTrue(g != g1)
        self.assertTrue(g.vcount() == g1.vcount() and g.ecount() == g1.ecount())
        self.assertTrue(g.is_directed() == g1.is_directed())
        self.assertTrue(g.get_edgelist() == g1.get_edgelist())

    def testUnion(self):
        g = Graph.Tree(7, 2) | Graph.Lattice([7])
        self.assertTrue(g.vcount() == 7 and g.ecount() == 12)
        self.assertTrue(sorted(g.get_edgelist()) == [
            (0, 1), (0, 2), (0, 6), (1, 2), (1, 3), (1, 4), (2, 3), (2, 5),
            (2, 6), (3, 4), (4, 5), (5, 6)
        ])

    def testUnionMethod(self):
        g = Graph.Tree(7, 2).union(Graph.Lattice([7]))
        self.assertTrue(g.vcount() == 7 and g.ecount() == 12)

    def testUnionNoGraphs(self):
        self.assertRaises(ValueError, union, [])

    def testUnionSingle(self):
        g1 = Graph.Tree(7, 2)
        g = union([g1])
        self.assertTrue(g != g1)
        self.assertTrue(g.vcount() == g1.vcount() and g.ecount() == g1.ecount())
        self.assertTrue(g.is_directed() == g1.is_directed())
        self.assertTrue(g.get_edgelist() == g1.get_edgelist())

    def testUnionMany(self):
        gs = [Graph.Tree(7, 2), Graph.Lattice([7]), Graph.Lattice([7])]
        g = union(gs)
        self.assertTrue(g.vcount() == 7 and g.ecount() == 12)

    def testUnionManyAttributes(self):
        gs = [
            Graph.Formula('A-B'),
            Graph.Formula('A-B,C-D'),
            ]
        gs[0]['attr'] = 'graph1'
        gs[0].vs['attr'] = ['set', 'set_too']
        gs[0].vs['attr2'] = ['set', 'set_too']
        gs[1].vs[0]['attr'] = 'set'
        gs[1].vs[0]['attr2'] = 'conflict'
        g = union(gs)
        names = g.vs['name']
        self.assertTrue(g['attr'] == 'graph1')
        self.assertTrue(g.vs[names.index('A')]['attr'] == 'set')
        self.assertTrue(g.vs[names.index('B')]['attr'] == 'set_too')
        self.assertTrue(g.ecount() == 2)
        self.assertTrue(sorted(g.vertex_attributes()) == ['attr', 'attr2_1', 'attr2_2', 'name'])

    def testUnionManyEdgemap(self):
        gs = [
            Graph.Formula('A-B'),
            Graph.Formula('C-D, A-B'),
            ]
        gs[0].es[0]['attr'] = 'set'
        gs[1].es[0]['attr'] = 'set_too'
        g = union(gs)
        for e in g.es:
            vnames = [g.vs[e.source]['name'], g.vs[e.target]['name']]
            if set(vnames) == set(['A', 'B']):
                self.assertTrue(e['attr'] == 'set')
            else:
                self.assertTrue(e['attr'] == 'set_too')

    def testIntersectionNoGraphs(self):
        self.assertRaises(ValueError, intersection, [])

    def testIntersectionSingle(self):
        g1 = Graph.Tree(7, 2)
        g = intersection([g1])
        self.assertTrue(g != g1)
        self.assertTrue(g.vcount() == g1.vcount() and g.ecount() == g1.ecount())
        self.assertTrue(g.is_directed() == g1.is_directed())
        self.assertTrue(g.get_edgelist() == g1.get_edgelist())

    def testIntersectionMany(self):
        gs = [Graph.Tree(7, 2), Graph.Lattice([7])]
        g = intersection(gs)
        self.assertTrue(g.get_edgelist() == [(0, 1)])

    def testIntersectionManyAttributes(self):
        gs = [Graph.Tree(7, 2), Graph.Lattice([7])]
        gs[0]['attr'] = 'graph1'
        gs[0].vs['name'] = ['one', 'two', 'three', 'four', 'five', 'six', '7']
        gs[1].vs['name'] = ['one', 'two', 'three', 'four', 'five', 'six', '7']
        gs[0].vs[0]['attr'] = 'set'
        gs[1].vs[5]['attr'] = 'set_too'
        g = intersection(gs)
        names = g.vs['name']
        self.assertTrue(g['attr'] == 'graph1')
        self.assertTrue(g.vs[names.index('one')]['attr'] == 'set')
        self.assertTrue(g.vs[names.index('six')]['attr'] == 'set_too')
        self.assertTrue(g.ecount() == 1)
        self.assertTrue(
            set(g.get_edgelist()[0]) == set([names.index('one'), names.index('two')]),
        )

    def testIntersectionManyEdgemap(self):
        gs = [
            Graph.Formula('A-B'),
            Graph.Formula('A-B,C-D'),
            ]
        gs[0].es[0]['attr'] = 'set'
        gs[1].es[1]['attr'] = 'set_too'
        g = intersection(gs)
        self.assertTrue(g.es['attr'] == ['set'])

    def testInPlaceAddition(self):
        g = Graph.Full(3)
        orig = g

        # Adding vertices
        g += 2
        self.assertTrue(g.vcount() == 5 and g.ecount() == 3
                        and g.clusters().membership == [0,0,0,1,2])

        # Adding a vertex by name
        g += "spam"
        self.assertTrue(g.vcount() == 6 and g.ecount() == 3
                        and g.clusters().membership == [0,0,0,1,2,3])

        # Adding a single edge
        g += (2, 3)
        self.assertTrue(g.vcount() == 6 and g.ecount() == 4
                        and g.clusters().membership == [0,0,0,0,1,2])

        # Adding two edges
        g += [(3, 4), (2, 4), (4, 5)]
        self.assertTrue(g.vcount() == 6 and g.ecount() == 7
                        and g.clusters().membership == [0]*6)

        # Adding two more vertices
        g += ["eggs", "bacon"]
        self.assertEqual(g.vs["name"], [None, None, None, None, None,
            "spam", "eggs", "bacon"])

        # Did we really use the original graph so far?
        # TODO: disjoint union should be modified so that this assertion
        # could be moved to the end
        self.assertTrue(id(g) == id(orig))

        # Adding another graph
        g += Graph.Full(3)
        self.assertTrue(g.vcount() == 11 and g.ecount() == 10
                        and g.clusters().membership == [0,0,0,0,0,0,1,2,3,3,3])

        # Adding two graphs
        g += [Graph.Full(3), Graph.Full(2)]
        self.assertTrue(g.vcount() == 16 and g.ecount() == 14
                        and g.clusters().membership == [0,0,0,0,0,0,1,2,3,3,3,4,4,4,5,5])

    def testAddition(self):
        g0 = Graph.Full(3)

        # Adding vertices
        g = g0+2
        self.assertTrue(g.vcount() == 5 and g.ecount() == 3
                        and g.clusters().membership == [0,0,0,1,2])
        g0 = g

        # Adding vertices by name
        g = g0+"spam"
        self.assertTrue(g.vcount() == 6 and g.ecount() == 3
                        and g.clusters().membership == [0,0,0,1,2,3])
        g0 = g

        # Adding a single edge
        g = g0+(2,3)
        self.assertTrue(g.vcount() == 6 and g.ecount() == 4
                        and g.clusters().membership == [0,0,0,0,1,2])
        g0 = g

        # Adding two edges
        g = g0+[(3, 4), (2, 4), (4, 5)]
        self.assertTrue(g.vcount() == 6 and g.ecount() == 7
                        and g.clusters().membership == [0]*6)
        g0 = g

        # Adding another graph
        g = g0+Graph.Full(3)
        self.assertTrue(g.vcount() == 9 and g.ecount() == 10
                        and g.clusters().membership == [0,0,0,0,0,0,1,1,1])

    def testInPlaceSubtraction(self):
        g = Graph.Full(8)
        orig = g

        # Deleting a vertex by vertex selector
        g -= 7
        self.assertTrue(g.vcount() == 7 and g.ecount() == 21
                        and g.clusters().membership == [0,0,0,0,0,0,0])

        # Deleting a vertex
        g -= g.vs[6]
        self.assertTrue(g.vcount() == 6 and g.ecount() == 15
                        and g.clusters().membership == [0,0,0,0,0,0])

        # Deleting two vertices
        g -= [4, 5]
        self.assertTrue(g.vcount() == 4 and g.ecount() == 6
                        and g.clusters().membership == [0,0,0,0])

        # Deleting an edge
        g -= (1, 2)
        self.assertTrue(g.vcount() == 4 and g.ecount() == 5
                        and g.clusters().membership == [0,0,0,0])

        # Deleting three more edges
        g -= [(1, 3), (0, 2), (0, 3)]
        self.assertTrue(g.vcount() == 4 and g.ecount() == 2
                        and g.clusters().membership == [0,0,1,1])

        # Did we really use the original graph so far?
        self.assertTrue(id(g) == id(orig))

        # Subtracting a graph
        g2 = Graph.Tree(3, 2)
        g -= g2
        self.assertTrue(g.vcount() == 4 and g.ecount() == 1
                        and g.clusters().membership == [0,1,2,2])

    def testNonzero(self):
        self.assertTrue(Graph(1))
        self.assertFalse(Graph(0))

    def testLength(self):
        self.assertRaises(TypeError, len, Graph(15))
        self.assertTrue(len(Graph(15).vs) == 15)
        self.assertTrue(len(Graph.Full(5).es) == 10)

    def testSimplify(self):
        el = [(0,1), (1,0), (1,2), (2,3), (2,3), (2,3), (3,3)]
        g = Graph(el)
        g.es["weight"] = [1, 2, 3, 4, 5, 6, 7]

        g2 = g.copy()
        g2.simplify()
        self.assertTrue(g2.vcount() == g.vcount())
        self.assertTrue(g2.ecount() == 3)

        g2 = g.copy()
        g2.simplify(loops=False)
        self.assertTrue(g2.vcount() == g.vcount())
        self.assertTrue(g2.ecount() == 4)

        g2 = g.copy()
        g2.simplify(multiple=False)
        self.assertTrue(g2.vcount() == g.vcount())
        self.assertTrue(g2.ecount() == g.ecount() - 1)

    def testContractVertices(self):
        g = Graph.Full(4) + Graph.Full(4) + [(0, 5), (1, 4)]

        g2 = g.copy()
        g2.contract_vertices([0, 1, 2, 3, 1, 0, 4, 5])
        self.assertEqual(g2.vcount(), 6)
        self.assertEqual(g2.ecount(), g.ecount())
        self.assertEqual(sorted(g2.get_edgelist()),
                [(0, 0), (0, 1), (0, 1), (0, 2), (0, 3), (0, 4), (0, 5),
                 (1, 1), (1, 2), (1, 3), (1, 4), (1, 5), (2, 3), (4, 5)])

        g2 = g.copy()
        g2.contract_vertices([0, 1, 2, 3, 1, 0, 6, 7])
        self.assertEqual(g2.vcount(), 8)
        self.assertEqual(g2.ecount(), g.ecount())
        self.assertEqual(sorted(g2.get_edgelist()),
                [(0, 0), (0, 1), (0, 1), (0, 2), (0, 3), (0, 6), (0, 7),
                 (1, 1), (1, 2), (1, 3), (1, 6), (1, 7), (2, 3), (6, 7)])

        g2 = Graph(10)
        g2.contract_vertices([0, 0, 1, 1, 2, 2, 3, 3, 4, 4])
        self.assertEqual(g2.vcount(), 5)
        self.assertEqual(g2.ecount(), 0)

    @skipIf(np is None, "test case depends on NumPy")
    def testContractVerticesWithNumPyIntegers(self):
        g = Graph.Full(4) + Graph.Full(4) + [(0, 5), (1, 4)]
        g2 = g.copy()
        g2.contract_vertices([np.int32(x) for x in [0, 1, 2, 3, 1, 0, 6, 7]])
        self.assertEqual(g2.vcount(), 8)
        self.assertEqual(g2.ecount(), g.ecount())
        self.assertEqual(sorted(g2.get_edgelist()),
                [(0, 0), (0, 1), (0, 1), (0, 2), (0, 3), (0, 6), (0, 7),
                 (1, 1), (1, 2), (1, 3), (1, 6), (1, 7), (2, 3), (6, 7)])


def suite():
    operator_suite = unittest.makeSuite(OperatorTests)
    return unittest.TestSuite([operator_suite])

def test():
    runner = unittest.TextTestRunner()
    runner.run(suite())

if __name__ == "__main__":
    test()




#usr/bin/env python

import os
import platform
import sys

###########################################################################

# Check Python's version info and exit early if it is too old
if sys.version_info < (2, 7):
    print("This module requires Python >= 2.7")
    sys.exit(0)

# Check whether we are compiling for PyPy. Headers will not be installed
# for PyPy.
SKIP_HEADER_INSTALL = (platform.python_implementation() == "PyPy") or (
    "SKIP_HEADER_INSTALL" in os.environ
)

###########################################################################

from setuptools import setup, Command, Extension

import distutils.ccompiler
import glob
import shutil
import subprocess
import sys
import sysconfig

from select import select

###########################################################################

LIBIGRAPH_FALLBACK_INCLUDE_DIRS = ["/usr/include/igraph", "/usr/local/include/igraph"]
LIBIGRAPH_FALLBACK_LIBRARIES = ["igraph"]
LIBIGRAPH_FALLBACK_LIBRARY_DIRS = []

###########################################################################


def building_on_windows_msvc():
    """Returns True when using the non-MinGW CPython interpreter on Windows"""
    return platform.system() == "Windows" and sysconfig.get_platform() != "mingw"


def create_dir_unless_exists(*args):
    """Creates a directory unless it exists already."""
    path = os.path.join(*args)
    if not os.path.isdir(path):
        os.makedirs(path)


def ensure_dir_does_not_exist(*args):
    """Ensures that the given directory does not exist."""
    path = os.path.join(*args)
    if os.path.isdir(path):
        shutil.rmtree(path)


def exclude_from_list(items, items_to_exclude):
    """Excludes certain items from a list, keeping the original order of
    the remaining items."""
    itemset = set(items_to_exclude)
    return [item for item in items if item not in itemset]


def find_static_library(library_name, library_path):
    """Given the raw name of a library in `library_name`, tries to find a
    static library with this name in the given `library_path`. `library_path`
    is automatically extended with common library directories on Linux and Mac
    OS X."""

    variants = ["lib{0}.a", "{0}.a", "{0}.lib", "lib{0}.lib"]
    if is_unix_like():
        extra_libdirs = [
            "/usr/local/lib64",
            "/usr/local/lib",
            "/usr/lib64",
            "/usr/lib",
            "/lib64",
            "/lib",
        ]
    else:
        extra_libdirs = []

    for path in extra_libdirs:
        if path not in library_path and os.path.isdir(path):
            library_path.append(path)

    for path in library_path:
        for variant in variants:
            full_path = os.path.join(path, variant.format(library_name))
            if os.path.isfile(full_path):
                return full_path


def first(iterable):
    """Returns the first element from the given iterable."""
    for item in iterable:
        return item
    raise ValueError("iterable is empty")


def get_output(args, encoding="utf-8"):
    """Returns the output of a command returning a single line of output."""
    PIPE = subprocess.PIPE
    try:
        p = subprocess.Popen(args, shell=False, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        stdout, stderr = p.communicate()
        returncode = p.returncode
    except OSError:
        stdout, stderr = None, None
        returncode = 77
    if encoding and type(stdout).__name__ == "bytes":
        stdout = str(stdout, encoding=encoding)
    if encoding and type(stderr).__name__ == "bytes":
        stderr = str(stderr, encoding=encoding)
    return stdout, returncode


def get_output_single_line(args, encoding="utf-8"):
    """Returns the output of a command returning a single line of output,
    stripped from any trailing newlines."""
    stdout, returncode = get_output(args, encoding=encoding)
    if stdout is not None:
        line, _, _ = stdout.partition("\n")
    else:
        line = None
    return line, returncode


def is_unix_like(platform=None):
    """Returns whether the given platform is a Unix-like platform with the usual
    Unix filesystem. When the parameter is omitted, it defaults to ``sys.platform``
    """
    platform = platform or sys.platform
    platform = platform.lower()
    return (
        platform.startswith("linux")
        or platform.startswith("darwin")
        or platform.startswith("cygwin")
    )


def find_msvc_source_folder(folder = ".", requires_built=False):
    """Finds the folder that contains the MSVC-specific source of igraph if there
    is any. Returns `None` if no such folder is found. Prints a warning if the
    choice is ambiguous.
    """
    all_msvc_dirs = glob.glob(os.path.join(folder, "igraph-*-msvc"))
    if len(all_msvc_dirs) > 0:
        if len(all_msvc_dirs) > 1:
            print(
                "More than one MSVC build directory (..\\..\\igraph-*-msvc) found!"
            )
            print(
                "It could happen that setup.py uses the wrong one! Please remove all but the right one!\n\n"
            )

        msvc_builddir = all_msvc_dirs[-1]
        if requires_built and not os.path.exists(os.path.join(msvc_builddir, "Release")):
            print(
                "There is no 'Release' dir in the MSVC build directory\n(%s)"
                % msvc_builddir
            )
            print("Please build the MSVC build first!\n")
            return None

        return msvc_builddir
    else:
        return None


def preprocess_fallback_config():
    """Preprocesses the fallback include and library paths depending on the
    platform."""
    global LIBIGRAPH_FALLBACK_INCLUDE_DIRS
    global LIBIGRAPH_FALLBACK_LIBRARY_DIRS
    global LIBIGRAPH_FALLBACK_LIBRARIES

    if platform.system() == "Windows" and distutils.ccompiler.get_default_compiler() == "msvc":
        # if this setup is run in the source checkout *and* the igraph msvc was build,
        # this code adds the right library and include dir
        msvc_builddir = find_msvc_source_folder(os.path.join("..", ".."), requires_built=True)

        if msvc_builddir is not None:
            print("Using MSVC build dir: %s\n\n" % msvc_builddir)
            LIBIGRAPH_FALLBACK_INCLUDE_DIRS = [
                os.path.join(msvc_builddir, "include")
            ]
            LIBIGRAPH_FALLBACK_LIBRARY_DIRS = [
                os.path.join(msvc_builddir, "Release")
            ]
            return True
        else:
            return False

    else:
        return True


def quote_path_for_shell(s):
    # On MinGW / MSYS, we need to use forward slash style and remove unsafe
    # characters in order not to trip up the configure script
    if "MSYSTEM" in os.environ:
        s = s.replace("\\", "/")
        if s[1:3] == ":/":
            s = "/" + s[0] + s[2:]

    # Now the proper quoting
    return "'" + s.replace("'", "'\\''") + "'"


def wait_for_keypress(seconds):
    """Wait for a keypress or until the given number of seconds have passed,
    whichever happens first.
    """
    is_windows = platform.system() == "windows"

    while seconds > 0:
        if seconds > 1:
            plural = "s"
        else:
            plural = ""

        sys.stdout.write(
            "\rContinuing in %2d second%s; press Enter to continue "
            "immediately. " % (seconds, plural)
        )
        sys.stdout.flush()

        if is_windows:
            from msvcrt import kbhit

            for i in range(10):
                if kbhit():
                    seconds = 0
                    break
                sleep(0.1)
        else:
            rlist, _, _ = select([sys.stdin], [], [], 1)
            if rlist:
                sys.stdin.readline()
                seconds = 0
                break

        seconds -= 1

    sys.stdout.write("\r" + " " * 65 + "\r")


###########################################################################


class IgraphCCoreBuilder(object):
    """Class responsible for downloading and building the C core of igraph
    if it is not installed yet."""

    def compile_in(self, build_folder, source_folder=None):
        """Compiles igraph from its source code in the given folder.

        source_folder is the name of the folder that contains igraph's source
        files. If it is `None`, it is assumed that it is the same as the
        build folder.
        """
        if source_folder is None:
            source_folder = build_folder

        source_folder = os.path.abspath(source_folder)
        build_folder = os.path.abspath(build_folder)
        
        build_to_source_folder = os.path.relpath(source_folder, build_folder)

        cwd = os.getcwd()
        try:
            os.chdir(source_folder)

            # Run the bootstrap script if we have downloaded a tarball from
            # Github
            if os.path.isfile("bootstrap.sh") and not os.path.isfile("configure"):
                print("Bootstrapping igraph...")
                retcode = subprocess.call("sh bootstrap.sh", shell=True)
                if retcode:
                    return False

            # Patch ltmain.sh so it does not freak out when the build directory
            # contains spaces
            with open("ltmain.sh") as infp:
                with open("ltmain.sh.new", "w") as outfp:
                    for line in infp:
                        if line.endswith("cd $darwin_orig_dir\n"):
                            line = line.replace(
                                "cd $darwin_orig_dir\n", 'cd "$darwin_orig_dir"\n'
                            )
                        outfp.write(line)
            shutil.move("ltmain.sh.new", "ltmain.sh")

            create_dir_unless_exists(build_folder)
            os.chdir(build_folder)

            print("Configuring igraph...")
            configure_args = ["--disable-tls", "--enable-silent-rules"]
            if "IGRAPH_EXTRA_CONFIGURE_ARGS" in os.environ:
                configure_args.extend(os.environ["IGRAPH_EXTRA_CONFIGURE_ARGS"].split(" "))
            retcode = subprocess.call(
                "sh {0} {1}".format(
                    quote_path_for_shell(os.path.join(build_to_source_folder, "configure")),
                    " ".join(configure_args)
                ),
                env=self.enhanced_env(CFLAGS="-fPIC", CXXFLAGS="-fPIC"),
                shell=True
            )
            if retcode:
                return False

            building_on_windows = building_on_windows_msvc()
            
            if building_on_windows:
                print("Creating Microsoft Visual Studio project...")
                retcode = subprocess.call("make msvc", shell=True)
                if retcode:
                    return False

            print("Building igraph...")
            if building_on_windows:
                msvc_source = find_msvc_source_folder()
                if not msvc_source:
                    return False

                devenv = os.environ.get("DEVENV_EXECUTABLE")
                os.chdir(msvc_source)
                if devenv is None:
                    retcode = subprocess.call("devenv /upgrade igraph.vcproj", shell=True)
                else:
                    retcode = subprocess.call([devenv, "/upgrade", "igraph.vcproj"])
                if retcode:
                    return False

                retcode = subprocess.call("msbuild.exe igraph.vcxproj /p:configuration=Release")
            else:
                retcode = subprocess.call("make", shell=True)

            if retcode:
                return False

            if building_on_windows:
                libraries = ["igraph"]
            else:
                libraries = []
                for line in open("igraph.pc"):
                    if line.startswith("Libs: ") or line.startswith("Libs.private: "):
                        words = line.strip().split()
                        libraries.extend(
                            word[2:] for word in words if word.startswith("-l")
                        )

                if not libraries:
                    # Educated guess
                    libraries = ["igraph"]

            return libraries

        finally:
            os.chdir(cwd)

    def copy_build_artifacts(
        self, source_folder, build_folder, install_folder, libraries
    ):
        building_on_windows = building_on_windows_msvc()
            
        create_dir_unless_exists(install_folder)

        ensure_dir_does_not_exist(install_folder, "include")
        ensure_dir_does_not_exist(install_folder, "lib")

        shutil.copytree(
            os.path.join(source_folder, "include"),
            os.path.join(install_folder, "include"),
        )
        create_dir_unless_exists(install_folder, "lib")

        for fname in glob.glob(os.path.join(build_folder, "include", "*.h")):
            shutil.copy(fname, os.path.join(install_folder, "include"))

        if building_on_windows:
            msvc_builddir = find_msvc_source_folder(build_folder, requires_built=True)
            if msvc_builddir is not None:
                print("Using MSVC build dir: %s\n\n" % msvc_builddir)
                for fname in glob.glob(
                    os.path.join(msvc_builddir, "Release", "*.lib")
                ):
                    shutil.copy(fname, os.path.join(install_folder, "lib"))
            else:
                print("Cannot find MSVC build dir in %s\n\n" % build_folder)
                return False
        else:
            for fname in glob.glob(
                os.path.join(build_folder, "src", ".libs", "libigraph.*")
            ):
                shutil.copy(fname, os.path.join(install_folder, "lib"))

        with open(os.path.join(install_folder, "build.cfg"), "w") as f:
            f.write(repr(libraries))

        return True

    @staticmethod
    def enhanced_env(**kwargs):
        env = os.environ.copy()
        for k, v in kwargs.items():
            prev = os.environ.get(k)
            env[k] = "{0} {1}".format(prev, v) if prev else v
        return env


class BuildConfiguration(object):
    def __init__(self):
        self.include_dirs = []
        self.library_dirs = []
        self.runtime_library_dirs = []
        self.libraries = []
        self.extra_compile_args = []
        self.extra_link_args = []
        self.define_macros = []
        self.extra_objects = []
        self.static_extension = False
        self.use_pkgconfig = False
        self._has_pkgconfig = None
        self.excluded_include_dirs = []
        self.excluded_library_dirs = []
        self.wait = platform.system() != "Windows"

    @property
    def has_pkgconfig(self):
        """Returns whether ``pkg-config`` is available on the current system
        and it knows about igraph or not."""
        if self._has_pkgconfig is None:
            if self.use_pkgconfig:
                line, exit_code = get_output_single_line(["pkg-config", "igraph"])
                self._has_pkgconfig = exit_code == 0
            else:
                self._has_pkgconfig = False
        return self._has_pkgconfig

    @property
    def build_c_core(self):
        """Returns a class representing a custom setup.py command that builds
        the C core of igraph.

        This is used in CI environments where we want to build the C core of
        igraph once and then build the Python interface for various Python
        versions without having to recompile the C core all the time.

        If is also used as a custom building block of `build_ext`.
        """

        buildcfg = self

        class build_c_core(Command):
            description = "Compile the C core of igraph only"
            user_options = []

            def initialize_options(self):
                pass

            def finalize_options(self):
                pass

            def run(self):
                buildcfg.c_core_built = buildcfg.compile_igraph_from_vendor_source()

        return build_c_core

    @property
    def build_ext(self):
        """Returns a class that can be used as a replacement for the
        ``build_ext`` command in ``setuptools`` and that will compile the C core
        of igraph before compiling the Python extension.
        """
        from setuptools.command.build_ext import build_ext
        from distutils.sysconfig import get_python_inc

        buildcfg = self

        class custom_build_ext(build_ext):
            def run(self):
                # Bail out if we don't have the Python include files
                include_dir = get_python_inc()
                if not os.path.isfile(os.path.join(include_dir, "Python.h")):
                    print("You will need the Python headers to compile this extension.")
                    sys.exit(1)

                # Check whether the user asked us to discover a pre-built igraph
                # with pkg-config
                detected = False
                if buildcfg.use_pkgconfig:
                    detected = buildcfg.detect_from_pkgconfig()
                    if not detected:
                        print("Cannot find the C core of igraph on this system using pkg-config.")
                        sys.exit(1)
                else:
                    # Build the C core from the vendored igraph source
                    self.run_command("build_c_core")
                    if not buildcfg.c_core_built:
                        # Fall back to an educated guess if everything else failed
                        if not detected:
                            buildcfg.use_educated_guess()

                # Add any extra library paths if needed; this is needed for the
                # Appveyor CI build
                if "IGRAPH_EXTRA_LIBRARY_PATH" in os.environ:
                    buildcfg.library_dirs = list(
                        os.environ["IGRAPH_EXTRA_LIBRARY_PATH"].split(os.pathsep)
                    ) + buildcfg.library_dirs
                
                # Replaces library names with full paths to static libraries
                # where possible. libm.a is excluded because it caused problems
                # on Sabayon Linux where libm.a is probably not compiled with
                # -fPIC
                if buildcfg.static_extension:
                    if buildcfg.static_extension == "only_igraph":
                        buildcfg.replace_static_libraries(only=["igraph"])
                    else:
                        buildcfg.replace_static_libraries(exclusions=["m"])

                # Prints basic build information
                buildcfg.print_build_info()

                # Find the igraph extension and configure it with the settings
                # of this build configuration
                ext = first(
                    extension
                    for extension in self.extensions
                    if extension.name == "igraph._igraph"
                )
                buildcfg.configure(ext)

                # Run the original build_ext command
                build_ext.run(self)

        return custom_build_ext

    @property
    def sdist(self):
        """Returns a class that can be used as a replacement for the
        ``sdist`` command in ``setuptools`` and that will clean up
        ``vendor/source/igraph`` before running the original ``sdist``
        command.
        """
        from setuptools.command.sdist import sdist
        from distutils.sysconfig import get_python_inc

        buildcfg = self

        class custom_sdist(sdist):
            def run(self):
                # Clean up vendor/source/igraph with git
                cwd = os.getcwd()
                try:
                    os.chdir(os.path.join("vendor", "source", "igraph"))
                    if os.path.exists(".git"):
                        retcode = subprocess.call("git clean -dfx", shell=True)
                        if retcode:
                            print("Failed to clean vendor/source/igraph with git")
                            print("")
                            return False
                finally:
                    os.chdir(cwd)

                # Run the original sdist command
                sdist.run(self)

        return custom_sdist

    def compile_igraph_from_vendor_source(self):
        """Compiles igraph from the vendored source code inside `vendor/igraph/source`.
        This folder typically comes from a git submodule.
        """
        if os.path.exists(os.path.join("vendor", "install", "igraph")):
            # Vendored igraph already compiled and installed, just use it
            self.use_vendored_igraph()
            return True

        vendor_source_path = os.path.join("vendor", "source", "igraph")
        if not os.path.isfile(os.path.join(vendor_source_path, "configure.ac")):
            # No git submodule present with vendored source
            print("Cannot find vendored igraph source in " + vendor_source_path)
            print("")
            return False

        source_folder = os.path.join("vendor", "source", "igraph")
        build_folder = os.path.join("vendor", "build", "igraph")
        install_folder = os.path.join("vendor", "install", "igraph")

        print("We are going to build the C core of igraph.")
        print("  Source folder: %s" % source_folder)
        print("  Build folder: %s" % build_folder)
        print("  Install folder: %s" % install_folder)
        print("")

        igraph_builder = IgraphCCoreBuilder()
        libraries = igraph_builder.compile_in(build_folder, source_folder=source_folder)
        if not libraries or not igraph_builder.copy_build_artifacts(
            source_folder=source_folder,
            build_folder=build_folder,
            install_folder=install_folder,
            libraries=libraries,
        ):
            print("Could not compile the C core of igraph.")
            print("")
            sys.exit(1)

        self.use_vendored_igraph()
        return True

    def configure(self, ext):
        """Configures the given Extension object using this build configuration."""
        ext.include_dirs = exclude_from_list(
            self.include_dirs, self.excluded_include_dirs
        )
        ext.library_dirs = exclude_from_list(
            self.library_dirs, self.excluded_library_dirs
        )
        ext.runtime_library_dirs = self.runtime_library_dirs
        ext.libraries = self.libraries
        ext.extra_compile_args = self.extra_compile_args
        ext.extra_link_args = self.extra_link_args
        ext.extra_objects = self.extra_objects
        ext.define_macros = self.define_macros

    def detect_from_pkgconfig(self):
        """Detects the igraph include directory, library directory and the
        list of libraries to link to using ``pkg-config``."""
        if not buildcfg.has_pkgconfig:
            return False

        cmd = ["pkg-config", "igraph", "--cflags", "--libs"]
        if self.static_extension:
            cmd += ["--static"]
        line, exit_code = get_output_single_line(cmd)
        if exit_code > 0 or len(line) == 0:
            return False

        opts = line.strip().split()
        self.libraries = [opt[2:] for opt in opts if opt.startswith("-l")]
        self.library_dirs = [opt[2:] for opt in opts if opt.startswith("-L")]
        self.include_dirs = [opt[2:] for opt in opts if opt.startswith("-I")]
        return True

    def print_build_info(self):
        """Prints the include and library path being used for debugging purposes."""
        if self.static_extension == "only_igraph":
            build_type = "dynamic extension with vendored igraph source"
        elif self.static_extension:
            build_type = "static extension"
        else:
            build_type = "dynamic extension"
        print("Build type: %s" % build_type)
        print("Include path: %s" % " ".join(self.include_dirs))
        if self.excluded_include_dirs:
            print("  - excluding: %s" % " ".join(self.excluded_include_dirs))
        print("Library path: %s" % " ".join(self.library_dirs))
        if self.excluded_library_dirs:
            print("  - excluding: %s" % " ".join(self.excluded_library_dirs))
        print("Runtime library path: %s" % " ".join(self.runtime_library_dirs))
        print("Linked dynamic libraries: %s" % " ".join(self.libraries))
        print("Linked static libraries: %s" % " ".join(self.extra_objects))
        print("Extra compiler options: %s" % " ".join(self.extra_compile_args))
        print("Extra linker options: %s" % " ".join(self.extra_link_args))

    def process_args_from_command_line(self):
        """Preprocesses the command line options before they are passed to
        setup.py and sets up the build configuration."""
        # Yes, this is ugly, but we don't want to interfere with setup.py's own
        # option handling
        opts_to_remove = []
        for idx, option in enumerate(sys.argv):
            if not option.startswith("--"):
                continue
            if option == "--static":
                opts_to_remove.append(idx)
                self.static_extension = True
            elif option == "--no-pkg-config":
                opts_to_remove.append(idx)
                self.use_pkgconfig = False
            elif option == "--no-wait":
                opts_to_remove.append(idx)
                self.wait = False
            elif option == "--use-pkg-config":
                opts_to_remove.append(idx)
                self.use_pkgconfig = True

        for idx in reversed(opts_to_remove):
            sys.argv[idx : (idx + 1)] = []

    def replace_static_libraries(self, only=None, exclusions=None):
        """Replaces references to libraries with full paths to their static
        versions if the static version is to be found on the library path."""
        building_on_windows = building_on_windows_msvc()
            
        if not building_on_windows and "stdc++" not in self.libraries:
            self.libraries.append("stdc++")

        if exclusions is None:
            exclusions = []

        for library_name in set(self.libraries) - set(exclusions):
            if only is not None and library_name not in only:
                continue

            static_lib = find_static_library(library_name, self.library_dirs)
            if static_lib:
                self.libraries.remove(library_name)
                self.extra_objects.append(static_lib)

    def use_vendored_igraph(self):
        """Assumes that igraph is installed already in ``vendor/install/igraph`` and sets up
        the include and library paths and the library names accordingly."""
        building_on_windows = building_on_windows_msvc()
            
        buildcfg.include_dirs = [os.path.join("vendor", "install", "igraph", "include")]
        buildcfg.library_dirs = [os.path.join("vendor", "install", "igraph", "lib")]
        if not buildcfg.static_extension:
            buildcfg.static_extension = "only_igraph"
            if building_on_windows:
                buildcfg.define_macros.append(("IGRAPH_STATIC", "1"))

        buildcfg_file = os.path.join("vendor", "install", "igraph", "build.cfg")
        if os.path.exists(buildcfg_file):
            buildcfg.libraries = eval(open(buildcfg_file).read())

    def use_educated_guess(self):
        """Tries to guess the proper library names, include and library paths
        if everything else failed."""

        preprocess_fallback_config()

        global LIBIGRAPH_FALLBACK_LIBRARIES
        global LIBIGRAPH_FALLBACK_INCLUDE_DIRS
        global LIBIGRAPH_FALLBACK_LIBRARY_DIRS

        print("WARNING: we were not able to detect where igraph is installed on")
        print("your machine (if it is installed at all). We will use the fallback")
        print("library and include paths hardcoded in setup.py and hope that the")
        print("C core of igraph is installed there.")
        print("")
        print("If the compilation fails and you are sure that igraph is installed")
        print("on your machine, adjust the following two variables in setup.py")
        print("accordingly and try again:")
        print("")
        print("- LIBIGRAPH_FALLBACK_INCLUDE_DIRS")
        print("- LIBIGRAPH_FALLBACK_LIBRARY_DIRS")
        print("")

        if self.wait:
            wait_for_keypress(seconds=10)

        self.libraries = LIBIGRAPH_FALLBACK_LIBRARIES[:]
        if self.static_extension:
            self.libraries.extend(["xml2", "z", "m", "stdc++"])
        self.include_dirs = LIBIGRAPH_FALLBACK_INCLUDE_DIRS[:]
        self.library_dirs = LIBIGRAPH_FALLBACK_LIBRARY_DIRS[:]


###########################################################################

# Import version number from version.py so we only need to change it in
# one place when a new release is created
__version__ = None
exec(open("src/igraph/version.py").read())

# Process command line options
buildcfg = BuildConfiguration()
buildcfg.process_args_from_command_line()

# Define the extension
sources = glob.glob(os.path.join("src", "_igraph", "*.c"))
igraph_extension = Extension("igraph._igraph", sources)

description = """Python interface to the igraph high performance graph
library, primarily aimed at complex network research and analysis.

Graph plotting functionality is provided by the Cairo library, so make
sure you install the Python bindings of Cairo if you want to generate
publication-quality graph plots. You can try either `pycairo
<http://cairographics.org/pycairo>`_ or `cairocffi <http://cairocffi.readthedocs.io>`_,
``cairocffi`` is recommended, in particular if you are on Python 3.x because
there were bug reports affecting igraph graph plots in Jupyter notebooks
when using ``pycairo`` (but not with ``cairocffi``).
"""

headers = ["src/_igraph/igraphmodule_api.h"] if not SKIP_HEADER_INSTALL else []

options = dict(
    name="python-igraph",
    version=__version__,
    url="https://igraph.org/python",
    description="High performance graph data structures and algorithms",
    long_description=description,
    license="GNU General Public License (GPL)",
    author="Tamas Nepusz",
    author_email="ntamas@gmail.com",
    ext_modules=[igraph_extension],
    package_dir={"igraph": "src/igraph"},
    packages=[
        "igraph",
        "igraph.app",
        "igraph.drawing",
        "igraph.remote"
    ],
    scripts=["scripts/igraph"],
    install_requires=["texttable>=1.6.2"],
    extras_require={
        "plotting": ["pycairo>=1.18.0"]
    },
    headers=headers,
    platforms="ALL",
    keywords=[
        "graph",
        "network",
        "mathematics",
        "math",
        "graph theory",
        "discrete mathematics",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Programming Language :: C",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Scientific/Engineering :: Mathematics",
        "Topic :: Scientific/Engineering :: Physics",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    cmdclass={
        "build_c_core": buildcfg.build_c_core,  # used by CI
        "build_ext": buildcfg.build_ext,
        "sdist": buildcfg.sdist
    },
)

if sys.version_info <= (2, 7):
    options["requires"] = options.pop("install_requires")

if sys.version_info > (3, 0):
    options["use_2to3"] = True

setup(**options)



#! /usr/bin/env python

import sys
import os.path
import re

from subprocess import check_output

# Some notes:
# - we have some sources with .cc extensions, these are marked as type 0
# - we have some non-standard header files, e.g. .hh, .pmt, these are type 2

srctext = """                       <File
                               RelativePath=".\src\%s"
                               FileType="0">
                       </File>"""

headtext = """                       <File
                               RelativePath=".\include\%s"
                               FileType="2">
                       </File>"""
headptext = """                       <File
                               RelativePath=".\src\%s"
                               FileType="2">
                       </File>"""

def runmake(makefile, target):
    out = check_output("make -q -C " + os.path.dirname(makefile) + 
                       " -f " + os.path.basename(makefile) + 
                       " " + target, shell=True)
    out = out.decode(sys.stdout.encoding or "utf8")
    out = out.replace("/", "\\")
    out = re.sub("make.*'.*'", "", out) # msys2 make adds "make[x]: entering '<dir>'""
    return out

def rreplace(s, old, new, occurrence):
    li = s.rsplit(old, occurrence)
    return new.join(li)

def main():
    if len(sys.argv) != 4:
        print( "Error: need three arguments")
        sys.exit(1)
    package = sys.argv[1]
    projectfile = sys.argv[2]
    makefile = sys.argv[3]

    proj = open(projectfile).read()
    sources = runmake(makefile, "echosources").split()
    headers = runmake(makefile, "echoheaders").split()
    headersprivate = runmake(makefile, "echoheadersprivate").split()

    # lex and bison stuff
    headers2 = [ rreplace(s, ".y", ".h",1) for s in sources if s[-2:]==".y" ]
    sources = [ rreplace(s, ".l", ".c", 1) for s in sources ]
    sources = [ rreplace(s, ".y", ".c", 1) for s in sources ]

    stext = "\n".join([ srctext % s for s in sources ])
    htext = "\n".join([ headtext % s for s in headers ])
    hptext = "\n".join([ headptext % s for s in headersprivate + headers2 ])

    proj = proj.replace("<!-- SOURCE-FILES -->", stext)
    proj = proj.replace("<!-- HEADER-FILES -->", htext + "\n" + hptext)

    out_file = open(package + "/igraph.vcproj", "w")
    out_file.write(proj)
    out_file.close()
    
if __name__ == "__main__":
    main()




import unittest

from igraph import *

def powerset(iterable):
    items_powers = [(item, 1 << i) for i, item in enumerate(iterable)]
    for i in range(1 << len(items_powers)):
        for item, power in items_powers:
            if i & power:
                yield item


leda_graph = Graph([
    (0,8),(0,12),(0,14),(1,9),(1,10),(1,13),
    (2,8),(2,9),(3,10),(3,11),(3,13),(4,9),(4,14),
    (5,14),(6,9),(6,14),(7,8),(7,12),(7,14)])
leda_graph.vs["type"] = [0]*8+[1]*7

class MatchingTests(unittest.TestCase):
    def setUp(self):
        self.matching = Matching(leda_graph,
                [12, 10, 8, 13, -1, 14, 9, -1, 2, 6, 1, -1, 0, 3, 5],
                "type")

    def testIsMaximal(self):
        self.assertTrue(self.matching.is_maximal())
        self.matching.matching[0] = -1
        self.matching.matching[12] = -1
        self.assertFalse(self.matching.is_maximal())

    def testMatchingRetrieval(self):
        m = [12, 10, 8, 13, -1, 14, 9, -1, 2, 6, 1, -1, 0, 3, 5]
        self.assertEqual(self.matching.matching, m)
        for i, mate in enumerate(m):
            if mate == -1:
                self.assertFalse(self.matching.is_matched(i))
                self.assertEqual(self.matching.match_of(i), None)
            else:
                self.assertTrue(self.matching.is_matched(i))
                self.assertEqual(self.matching.match_of(i), mate)
                self.assertEqual(self.matching.match_of(
                    leda_graph.vs[i]).index, leda_graph.vs[mate].index)


class MaximumBipartiteMatchingTests(unittest.TestCase):
    def testBipartiteMatchingSimple(self):
        # Specifying the "type" attribute explicitly
        matching = leda_graph.maximum_bipartite_matching("type")
        self.assertEqual(len(matching), 6)
        self.assertTrue(matching.is_maximal())

        # Using the default attribute
        matching = leda_graph.maximum_bipartite_matching()
        self.assertEqual(len(matching), 6)
        self.assertTrue(matching.is_maximal())

    def testBipartiteMatchingErrors(self):
        # Type vector too short
        g = Graph([(0, 1), (1, 2), (2, 3)])
        self.assertRaises(InternalError, g.maximum_bipartite_matching,
                types=[0,1,0])

        # Graph not bipartite
        self.assertRaises(InternalError, g.maximum_bipartite_matching,
                types=[0,1,1,1])


def suite():
    matching_suite = unittest.makeSuite(MatchingTests)
    bipartite_unweighted_suite = unittest.makeSuite(MaximumBipartiteMatchingTests)
    return unittest.TestSuite([matching_suite, bipartite_unweighted_suite])

def test():
    runner = unittest.TextTestRunner()
    runner.run(suite())
    
if __name__ == "__main__":
    test()

